'''
Emails.models
'''

from django.db import models
from Users.models import Person, PersonList
from basic.funcs import PidgeException


class EmailTemplate(models.Model):
    '''
    An email template for a message sent to people
    '''
    name = models.CharField(
        "name for the email",
        max_length = 128,
        help_text = "(required) Used to help you identify the mailing.")
    
    email_reply_to = models.EmailField(
        "relpy to address",
        help_text = "(required) Where you want replies to your email to go to. ")

    email_reply_to_name = models.CharField(
        "name of reply to address",
        max_length = 128,
        blank = True,
        help_text = "The name of the mailbox of the person who will receive replies.")
    
    email_from = models.EmailField(
        "sender address",
        help_text = "(required) The email address you want the email to appear to be from. ")

    email_from_name = models.CharField(
        "name of sender",
        max_length = 128,
        help_text = "(required) The name of the mailbox of the sender. If I were sending emails for MyPidge I'd use the name 'MyPidge.com'.")

    subject = models.CharField(
        "email subject line",
        max_length = 128,
        help_text = "(required) Try and keep it below 60 characters. Most email clients won't show more than this. ")
    
    body = models.TextField("email content")
    
    lists = models.ManyToManyField(
        PersonList,
        blank = True,
        help_text = "(required) The email lists used to send the email.")
    
    recipients = models.ManyToManyField(
        Person,
        through = 'EmailRecipient',
        blank = True,
        help_text = "The individuals who receive this. Different to lists, as the composition of lists may change over time.")

    class EmailTemplateError(PidgeException): pass

    def __unicode__(self):
        return u'%s: "%s"' % (self.name, self.subject)

    def send(self, preview = False):
        '''
        Does the business of sending the messages
        
        Set preview = True to be returned only a list of messages as they would be sent
        '''
        import datetime
        from basic.funcs import SendEmail
        from django.template import loader, Context, Template

        sender = '"%s" <%s>' % (self.email_from_name, self.email_from)
        subject_t = Template(self.subject)
        message_t = Template(self.body)
        headers = {'Reply-To': '"%s" <%s>' % (self.email_reply_to_name, self.email_reply_to) }
        
        # Only get recipients who have not been sent the message yet
        recipientset = EmailRecipient.objects.filter(email = self, sent__isnull = True)

        messages = []
        if preview:
            messages.append("Sending %s\nFrom: %s\nReply-To: %s\n\n\n" % (self.name, sender, headers['Reply-To']))
        else:
            messages.append("Attempted to send to %s people.\n" % recipientset.count())
        
        count = 0
        for recipient in recipientset:
            count += 1
            if count > 20:
                return messages
            # Render subject and body with a context
            c = Context({"person": recipient.person})
            subject = subject_t.render(c)
            message = message_t.render(c)
            
            # If we find curly braces now, we assume it's a typo in the email
            if subject.count("}") or subject.count("{"):
                raise EmailTemplate.EmailTemplateError, "Subject has stray curly braces."
            
            if message.count("}") or message.count("{"):
                raise EmailTemplate.EmailTemplateError, "Message body has stray curly braces."
            
            email_to = []
            if recipient.alternate_address:
                email_to = ["\"%s\" <%s>" % (unicode(recipient.person), recipient.alternate_address)]                
            else:    
                email_to = ["\"%s\" <%s>" % (unicode(recipient.person), recipient.person.email)]
            
            if preview:
                mymessage = "To: %s\nSubject: %s\n%s\n\n\n\n" % (email_to, subject, message)
                messages.append(mymessage)
                
            else:
                try:
                    SendEmail(subject = subject, sender = sender, message = message, to = email_to, headers = headers)
                except Exception, message:
                    messages.append(recipient.person.email + " " + str(message.args[0]))
                else:
                    recipient.sent = datetime.datetime.now()
                    recipient.save()
        
        return messages
    
    def add_recipient(self, person, alternate_email = ""):
        '''
        Takes a person and adds them as a queued recipient of this message.
        '''
        recip, created = EmailRecipient.objects.get_or_create(person = person, email = self, defaults= {'sent': None, 'alternate_address': alternate_email })
        return created
    
    def add_list(self, list):
        '''
        Takes a list as input and adds its recipients to this message.
        '''
        usersadded = []
        for person in list.people.all():
            recip, created = EmailRecipient.objects.get_or_create(person = person, email = self, defaults= {'sent': None})
            if created:
                person.added = True
            usersadded.append(person)
        return usersadded
        

class EmailRecipient(models.Model):
    '''
    A Person receiving an email.
    '''
    person = models.ForeignKey(Person)
    email = models.ForeignKey(EmailTemplate)
    alternate_address = models.EmailField(
        "email address actually used to send message",
        blank = True)
    sent = models.DateTimeField(null = True, blank = True)

    def __unicode__(self):
        return u'%s %s [%s]' % (unicode(self.email), unicode(self.person), self.sent)
