'''
Email.tests.models
'''

from Email.models import EmailTemplate, EmailRecipient
from Users.models import Person, PersonList
from django.core import mail
from django.test import TestCase

class EmailTemplateTests(TestCase):
    
    def setUp(self):
        self.emailtemplate = EmailTemplate.objects.create(
            name = "My Test List",
            email_reply_to = "frf21@cam.ac.uk",
            email_reply_to_name = "MyPidge.com",
            email_from = "fergus@mypidge.com",
            email_from_name = "MyPidge.com",
            subject = "{{ person.firstname }}, would you like it?",
            body = "You're never going to believe it, {{ person.firstname }}. We've only gone and got the mailing list system working!\n\nFergus Ross Ferrier")
         
        self.userone = Person.objects.create(
            firstname = "One",
            lastname = "Must",
            email = "one_must@thepidge.com"
            )

        self.usertwo = Person.objects.create(
            firstname = "Two",
            lastname = "Django",
            email = "two_django@thepidge.com"
            )

        self.userthree = Person.objects.create(
            firstname = "Three",
            lastname = "Company",
            email = "three_company@thepidge.com"
            )
            
        self.userlist = PersonList.objects.create(
            name = "My First List",
            )
            
        self.userlist.people = [self.userone, self.usertwo]
        
        self.anotheruserlist = PersonList.objects.create(
            name = "My Second List"
            )
        self.anotheruserlist.people = [self.userthree, self.usertwo]
        
        self.emptylist = PersonList.objects.create(
            name = "My Dull List"
            )
    
    def test_unicode_name(self):
        ''' Check unicode string rep of EmailTemplate '''
        self.failUnlessEqual(unicode(self.emailtemplate), 'My Test List: "{{ person.firstname }}, would you like it?"')

    # Add recipient
    
    def test_add_recipient(self):
        ''' Add a user to the template '''
        self.assertEquals(EmailRecipient.objects.filter(email = self.emailtemplate).count(), 0)
        self.emailtemplate.add_recipient(self.userone)
        self.assertEquals(EmailRecipient.objects.filter(email = self.emailtemplate).count(), 1)

    def test_add_recipient_alternate_email(self):
        ''' We can send the email to an email address other than the person's normal email address '''
        self.assertEquals(EmailRecipient.objects.filter(email = self.emailtemplate).count(), 0)
        alternate_email = "testthisemail@there.com"
        self.emailtemplate.add_recipient(self.userone, alternate_email = alternate_email)
        self.assertEquals(EmailRecipient.objects.filter(email = self.emailtemplate).count(), 1)
        self.assertEquals(EmailRecipient.objects.get(email = self.emailtemplate).alternate_address, alternate_email)
        
        
    def test_add_recipient_again(self):
        ''' A user can only be added to a template once. '''
        self.assertEquals(EmailRecipient.objects.filter(email = self.emailtemplate).count(), 0)
        self.emailtemplate.add_recipient(self.userone)
        self.emailtemplate.add_recipient(self.userone)
        self.assertEquals(EmailRecipient.objects.filter(email = self.emailtemplate).count(), 1)

    # Add list
    def test_add_list(self):
        ''' Turn a UserList of 2 people into 2 recipients for a mailing '''
        self.emailtemplate.add_list(self.userlist)
        self.assertEquals(EmailRecipient.objects.filter(email = self.emailtemplate).count(), 2)
    
    def test_add_empty_list(self):
        ''' Add an empty UserList to an EmailTemplate '''
        self.emailtemplate.add_list(self.emptylist)
        self.assertEquals(EmailRecipient.objects.filter(email = self.emailtemplate).count(), 0)
    
    def test_add_intersecting_lists(self):
        ''' Adding two UserLists (which have overlapping users) to EmailTemplate '''
        self.emailtemplate.add_list(self.userlist)
        self.emailtemplate.add_list(self.anotheruserlist)
        self.assertEquals(EmailRecipient.objects.filter(email = self.emailtemplate).count(), 3)
    
    # Send
    
    def test_send_none(self):
        ''' Sending EmailTemplate with no recipients. '''
        self.emailtemplate.send()
        self.assertEquals(len(mail.outbox), 0)
        
    def test_send(self):
        ''' Sending EmailTemplate to 3 recipients '''
        self.emailtemplate.add_list(self.userlist)
        self.emailtemplate.add_list(self.anotheruserlist)
        self.assertEquals(EmailRecipient.objects.filter(email = self.emailtemplate, sent__isnull = True).count(), 3)
        self.assertEquals(self.emailtemplate.send(), ["Attempted to send to 3 people.\n"])
        self.assertEquals(len(mail.outbox), 3)
        self.assertEquals(EmailRecipient.objects.filter(email = self.emailtemplate, sent__isnull = True).count(), 0)

        # Sending again does nothing
        self.emailtemplate.send()
        self.assertEquals(len(mail.outbox), 3)
        self.assertEquals(EmailRecipient.objects.filter(email = self.emailtemplate, sent__isnull = True).count(), 0)
        
    def test_send_template_leak(self):
        ''' Sending EmailTemplate with stray {}s in templates '''
        self.emailtemplate.subject = "{ person.firstname}} Hey there"
        self.emailtemplate.add_list(self.userlist)
        self.assertRaises(EmailTemplate.EmailTemplateError, self.emailtemplate.send)
    
    def test_send_alternate_email(self):
        ''' Sending template to Person with alternate email uses that email '''
        alternate_email = "testthisemail@there.com"
        self.emailtemplate.add_recipient(self.userone, alternate_email = alternate_email)
        self.emailtemplate.send()
        self.assertEquals(mail.outbox[0].to[0], '"One Must" <testthisemail@there.com>')
    
    # Preview
    
    def test_preview(self):
        ''' Preview send EmailTemplate gives proper emails '''
        self.emailtemplate.add_list(self.userlist)
        self.emailtemplate.add_list(self.anotheruserlist)
        myresponse = self.emailtemplate.send(preview = True)
        myresponse = '\n'.join(myresponse)
        self.assertEquals(myresponse.count("Subject: One, would you like it?"), 1)
        self.assertEquals(myresponse.count("Subject: Two, would you like it?"), 1)
        self.assertEquals(myresponse.count("Subject: Three, would you like it?"), 1)
        self.assertEquals(myresponse.count("You're never going to believe it, One."), 1)
        self.assertEquals(myresponse.count("You're never going to believe it, Two."), 1)
        self.assertEquals(myresponse.count("You're never going to believe it, Three."), 1)
        self.assertEquals(myresponse.count("From: \"MyPidge.com\" <fergus@mypidge.com>"), 1)
        self.assertEquals(myresponse.count("Reply-To: \"MyPidge.com\" <frf21@cam.ac.uk>"), 1)
    
    
class EmailRecipientTests(TestCase):
    
    def setUp(self):
        self.emailtemplate = EmailTemplate.objects.create(
            name = "My Test List",
            email_reply_to = "frf21@cam.ac.uk",
            email_reply_to_name = "MyPidge.com",
            email_from = "fergus@mypidge.com",
            email_from_name = "MyPidge.com",
            subject = "{{ person.firstname }}, would you like it?",
            body = "You're never going to believe it, {{ person.firstname }}. We've only gone and got the mailing list system working!\n\nFergus Ross Ferrier")
            
        self.user = Person.objects.create(
            firstname = "Fergus",
            lastname = "Ferrier",
            email = "ferg@thepidge.com"
            )
            
        self.recipient = EmailRecipient.objects.create(
            person = self.user,
            email = self.emailtemplate
            )
            
    def test_unicode_name(self):
        ''' Check unicode string rep of EmailRecipient '''
        self.failUnlessEqual(unicode(self.recipient), 'My Test List: "{{ person.firstname }}, would you like it?" Fergus Ferrier [None]')
