'''
Events.tests.views
'''

from Events.models import Event, EventTime, EventTag, Observance
from Groups.models import Group, GroupCategory
from Users.models import Reminder, User, Membership
from django.test import TestCase, Client
from django.core import mail
from django.core.urlresolvers import reverse
from datetime import datetime, timedelta

class BasicViewTests(TestCase):
    def setUp(self):
        # Reqd so that PREPEND_WWW in settings doesn't cause all view tests to fail
        self.host = "www.testhost" 
        self.client = Client(HTTP_HOST = self.host)
        
        self.category = GroupCategory.objects.create(
            name = "NewCat",
            small_name = "NC",
            all_can_create = True)
        
        self.group = Group.objects.create(
            official_name = "My Python",
            category = self.category)
        
        self.talkscam, created = Group.objects.get_or_create(
            official_name = "talks.cam",
            friendly_name = "talks.cam",
            defaults={'category': self.category})
        
        self.eventtag = EventTag.objects.create(name = "Fun Things")
        
        self.event = Event.objects.create(
            group = self.group,
            title = "Dull Old Event",
            description = "The best thing to have happened recently.")   
        
        self.eventtime = EventTime.objects.create(
            event = self.event,
            start = "2008-10-30 16:00:00",
            finish = "2008-10-30 18:00:00")
        
        self.eventfuture = Event.objects.create(
            group = self.group,
            title = "Exciting Upcoming Event",
            description = "The best thing to have happened recently.")   
        self.eventfuture.tags.add(self.eventtag)
        
        self.eventfuturetime = EventTime.objects.create(
            event = self.eventfuture,
            start = "2009-12-30 16:00:00",
            finish = "2009-12-30 18:00:00")
        
        self.eventfuturetalk = Event.objects.create(
            group = self.talkscam,
            title = "Exciting Upcoming Talk",
            description = "The best thing to have happened recently.")   
        
        self.eventfuturetalktime = EventTime.objects.create(
            event = self.eventfuturetalk,
            start = "2009-12-30 16:00:00",
            finish = "2009-12-30 18:00:00")

        # Event with no end time
        self.event_moment = Event.objects.create(
            group = self.group,
            title = "Momentary Event",
            description = "There is no end time.")   
        self.event_moment.tags.add(self.eventtag)
        self.event_moment_time = EventTime.objects.create(
            event = self.event_moment,
            start = "2009-12-30 16:00:00")

        # Event 15 minutes ago
        self.event_just_past = Event.objects.create(
            group = self.group,
            title = "Just Past Event",
            description = "Gosh you've JUST missed it.")   
        self.event_just_past.tags.add(self.eventtag)
        self.event_just_past_time = EventTime.objects.create(
            event = self.event_just_past,
            start = datetime.now() - timedelta(minutes = 15),
            finish = datetime.now() + timedelta(hours = 1))

    # Events Page
    
    def test_events(self):
        ''' Events page shows events in future only [with 15 min grace], but not talks '''
        response = self.client.get(reverse('Events.views.All'))
        self.assertContains(response, "Exciting Upcoming Event", count=1, status_code=200)
        self.assertContains(response, "Momentary Event", count=1, status_code=200)
        self.assertContains(response, "Just Past Event", count=1, status_code=200)
        self.assertNotContains(response, "Dull Old Event")
        self.assertNotContains(response, "Exciting Upcoming Talk")

    def test_events_none(self):
        ''' No events in future. [Let's hope this never happens] '''
        Event.objects.all().delete()
        response = self.client.get(reverse('Events.views.All'))
        self.assertContains(response, "There are no events to list. Perhaps select less restrictive options above.", count=1, status_code=200)
        
    def test_events_category(self):
        ''' Choosing a category of events to view. '''
        response = self.client.get(reverse('Events.views.All'), {'category': self.eventtag.id})
        self.assertContains(response, "Exciting Upcoming Event", count=1, status_code=200)
        self.assertNotContains(response, "Exciting Upcoming Talk")

    # View Event
    
    def test_event_bad(self):
        ''' view an event that doesn't exist '''
        response = self.client.get(reverse('Events.views.ViewEvent', args=[44444]))
        self.assertEquals(response.status_code, 404)
    
    
    def test_event(self):
        ''' View an event '''
        response = self.client.get(reverse('Events.views.ViewEvent', args=[self.event.id]))
        self.assertContains(response, "Dull Old Event", count=3, status_code=200)
        
    # Observances
    
    def test_observance(self):
        ''' Get HTML of day's observance '''
        response = self.client.get(reverse('Events.views.GetObservance'))
        self.assertEquals(response.status_code, 200)
        

class UserViewTests(TestCase):
    def setUp(self):
        # Reqd so that PREPEND_WWW in settings doesn't cause all view tests to fail
        self.host = "www.testhost" 
        self.client = Client(HTTP_HOST = self.host)
        
        self.category = GroupCategory.objects.create(
            name = "NewCat",
            small_name = "NC",
            all_can_create = True)
        
        self.group = Group.objects.create(
            official_name = "My Python",
            category = self.category)
        
        self.talkscam, created = Group.objects.get_or_create(
            official_name = "talks.cam",
            friendly_name = "talks.cam",
            defaults={'category': self.category})
        
        self.eventtag = EventTag.objects.create(name = "Fun Things")
        
        self.event = Event.objects.create(
            group = self.group,
            title = "Dull Old Event",
            description = "The best thing to have happened recently.")   
        
        self.eventtime = EventTime.objects.create(
            event = self.event,
            start = "2008-10-30 16:00:00",
            finish = "2008-10-30 18:00:00")
        
        self.eventfuture = Event.objects.create(
            group = self.group,
            title = "Exciting Upcoming Event",
            description = "The best thing to have happened recently.")   
        
        self.eventfuture.tags.add(self.eventtag)
        self.eventfuturetime = EventTime.objects.create(
            event = self.eventfuture,
            start = "2009-12-30 16:00:00",
            finish = "2009-12-30 18:00:00")
        
        self.eventfuturetalk = Event.objects.create(
            group = self.talkscam,
            title = "Exciting Upcoming Talk",
            description = "The best thing to have happened recently.")   
        
        self.eventfuturetalktime = EventTime.objects.create(
            event = self.eventfuturetalk,
            start = "2009-12-30 16:00:00",
            finish = "2009-12-30 18:00:00")
        
        self.fergus = User.objects.create(
            firstname="Fergus",
            lastname="Ferrier",
            email="fff21@cam.ac.uk",
            password = "blah",
            membershipstatus="OK",
            activationlinkkey="imokay")
        self.fergus.UpdatePassword("password")
        
        self.client.post(
            reverse('Users.views.Login', args=[""]),
            {'email': self.fergus.email, 'password': "password"})
    
    # Event page user functions
    
    def test_event_reminder_day(self):
        ''' View an event with the day reminder set '''
        Reminder.objects.create(person = self.fergus, event = self.event, assoctype='EML_DY')
        response = self.client.get(reverse('Events.views.ViewEvent', args=[self.event.id]))
        self.assertContains(response, "<input id='day' name='day' type='checkbox' checked value='1'/>", count=1, status_code=200)

    def test_event_reminder_hour(self):
        ''' View an event with the hour reminder set '''
        Reminder.objects.create(person = self.fergus, event = self.event, assoctype='EML_HR')
        response = self.client.get(reverse('Events.views.ViewEvent', args=[self.event.id]))
        self.assertContains(response, "<input id='hour' name='hour' type='checkbox' checked value='1'/>", count=1, status_code=200)        
    
    def test_event_controller(self):
        ''' View an event as controller '''
        response = self.client.get(reverse('Events.views.ViewEvent', args=[self.event.id]))
        self.assertNotContains(response, "Edit this Event", status_code=200)
        self.assertNotContains(response, "(delete)", status_code=200)
        self.assertNotContains(response, "Add another time for this event", status_code=200)
        Membership.objects.create(person = self.fergus, group = self.group, assoctype = "CTL")
        response = self.client.get(reverse('Events.views.ViewEvent', args=[self.event.id]))
        self.assertContains(response, "Edit this Event", status_code=200)
        self.assertContains(response, "(delete)", status_code=200)
        self.assertContains(response, "Add another time for this event", status_code=200)
    
    # Add an event
    
    def test_event_add(self):
        ''' Show the groups to which a user may add a group '''
        response = self.client.get(reverse('Events.views.Add'))
        self.assertEquals(response.status_code, 200)
        Membership.objects.create(person = self.fergus, group = self.group, assoctype = "CTL")
        response = self.client.get(reverse('Events.views.Add'))
        self.assertContains(response, "My Python", status_code=200)
    
    # Clone or Create New Event for Group
    
    def test_event_add_group(self):
        ''' Show the fresh or clone add event page '''
        Membership.objects.create(person = self.fergus, group = self.group, assoctype = "CTL")
        response = self.client.get(reverse('Events.views.AddOrClone', args=[self.group.id]))
        self.assertContains(response, "Dull Old Event", status_code=200)
        
        self.event.delete()
        self.eventfuture.delete()
        self.assertEquals(Event.objects.filter(group = self.group).count(), 0)
        
        # When no events to clone from a group, redirect straight to add details page
        responsex = self.client.get(reverse('Events.views.AddOrClone', args=[self.group.id]))
        self.assertEquals(responsex.content, "")
        self.assertRedirects(responsex, reverse('Events.views.AddToGroup', args=[self.group.id]), host = self.host)
        
        
    def test_event_add_group_bad(self):
        ''' Event add page when group does not exist '''
        response = self.client.get(reverse('Events.views.AddOrClone', args=[4444]))
        self.assertEquals(response.status_code, 404)
        
    def test_event_add_group_denied(self):
        ''' Event add page when user does not have group permissions '''
        response = self.client.get(reverse('Events.views.AddOrClone', args=[self.group.id]))
        self.assertEquals(response.status_code, 403)
    
    # Add Event to Group
    
    def test_event_add_to_group_bad(self):
        ''' Create an event from scratch for a group that does not exist '''
        response = self.client.get(reverse('Events.views.AddToGroup', args=[4444]))
        self.assertEquals(response.status_code, 404)    

    def test_event_add_to_group_denied(self):
        ''' Create an event from scratch for a group that does not exist '''
        response = self.client.get(reverse('Events.views.AddToGroup', args=[self.group.id]))
        self.assertEquals(response.status_code, 403)   
    
    def test_event_add_to_group(self):
        ''' Create an event from scratch for a group '''
        Membership.objects.create(person = self.fergus, group = self.group, assoctype = "CTL")
        response = self.client.get(reverse('Events.views.AddToGroup', args=[self.group.id]))
        self.assertEquals(response.status_code, 200)
    
    def test_event_add_to_group_clone(self):
        ''' Clone an event for a group '''
        Membership.objects.create(person = self.fergus, group = self.group, assoctype = "CTL")
        response = self.client.get(reverse('Events.views.AddToGroup', args=[self.group.id]), {'clone': self.event.id})
        self.assertContains(response, self.event.title, count = 1, status_code = 200)
        
    def test_event_add_to_group_post(self):
        ''' Submitting a new event for a group '''
        Membership.objects.create(person = self.fergus, group = self.group, assoctype = "CTL")
        response = self.client.post(reverse('Events.views.AddToGroup', args=[self.group.id]), {
            'title': "New Event",
            'description': "It's vonderful and newwwww.",
            'cost': "Zilch.",
            'location_additional': "Ztuff"})
        self.assertEquals(response.status_code, 302)
    
    # Edit Event
    
    def test_event_edit_bad(self):
        ''' Trying to edit an event that does not exist '''
        response = self.client.get(reverse('Events.views.EditEvent', args=[44444]))
        self.assertEquals(response.status_code, 404)

    def test_event_edit_denied(self):
        ''' Trying to edit an event for a group you don't have control over  '''
        response = self.client.get(reverse('Events.views.EditEvent', args=[self.event.id]))
        self.assertEquals(response.status_code, 403)

    def test_event_edit(self):
        ''' Trying to edit an event '''
        Membership.objects.create(person = self.fergus, group = self.group, assoctype = "CTL")
        response = self.client.get(reverse('Events.views.EditEvent', args=[self.event.id]))
        self.assertContains(response, self.event.title, status_code = 200)    
    
    def test_event_edit_post(self):
        ''' Updating an event '''
        Membership.objects.create(person = self.fergus, group = self.group, assoctype = "CTL")
        response = self.client.post(reverse('Events.views.EditEvent', args=[self.event.id]), {
            'title': 'New Title',
            'description': 'Newer than before. yes. ',
            'cost': 'Now free!',
            'location': 'here!',
            'tags': [self.eventtag.id]
            })
        self.assertRedirects(response, reverse('Events.views.ViewEvent', args=[self.event.id]), host = self.host)
    
    # Delete EventTime
    def test_event_time_delete_confirm(self):
        ''' Confirm that you want to remove an eventtime '''
        Membership.objects.create(person = self.fergus, group = self.group, assoctype = "CTL")
        response = self.client.get(reverse('Events.views.DeleteTime', args=[self.event.id, self.eventtime.id]))
        self.assertEquals(response.status_code, 200)
    
    def test_event_time_delete(self):
        ''' Remove an EventTime '''
        Membership.objects.create(person = self.fergus, group = self.group, assoctype = "CTL")
        response = self.client.post(reverse('Events.views.DeleteTime', args=[self.event.id, self.eventtime.id]), {'blank':'1'})
        self.assertRaises(EventTime.DoesNotExist, EventTime.objects.get, id = self.eventtime.id)
        self.assertRedirects(response, reverse('Events.views.ViewEvent', args=[self.event.id]), host = self.host)

        
    # Add EventTime
    
    def test_event_time_add(self):
        ''' Add an EventTime form shows '''
        Membership.objects.create(person = self.fergus, group = self.group, assoctype = "CTL")
        response = self.client.get(reverse('Events.views.AddTime', args=[self.event.id]))
        self.assertNotContains(response, "This event has been added.", status_code = 200)
        
    def test_event_time_added(self):
        ''' Adding an EventTime after adding the event '''
        Membership.objects.create(person = self.fergus, group = self.group, assoctype = "CTL")
        response = self.client.get(reverse('Events.views.NowAddTime', args=[self.event.id]))
        self.assertContains(response, "This event has been added.", count = 1, status_code = 200)
        
    def test_event_time_add_negative(self):
        ''' Adding an EventTime with finish before start! '''
        Membership.objects.create(person = self.fergus, group = self.group, assoctype = "CTL")
        response = self.client.post(reverse('Events.views.NowAddTime', args=[self.event.id]), {
            'start': '2008-12-31 10:20:11',
            'finish': '2008-12-30 12:00:00'})
        self.assertContains(response, "The finish time must be after the start time!", count = 1, status_code = 200)
        self.assertNotContains(response, "Saved these times for this event!", status_code = 200)

    def test_event_time_add_okay(self):
        ''' Adding an EventTime '''
        Membership.objects.create(person = self.fergus, group = self.group, assoctype = "CTL")
        response = self.client.post(reverse('Events.views.NowAddTime', args=[self.event.id]), {
            'start': '2008-12-31 10:20:11',
            'finish': '2008-12-31 12:00:00'})
        self.assertNotContains(response, "The finish time was before the start time! Please correct this and resubmit.", status_code = 200)
        self.assertContains(response, "Saved these times for this event!", status_code = 200)        

    def test_event_time_add_no_end(self):
        ''' Adding an EventTime with no end time'''
        Membership.objects.create(person = self.fergus, group = self.group, assoctype = "CTL")
        response = self.client.post(reverse('Events.views.NowAddTime', args=[self.event.id]), {
            'start': '2008-12-31 10:20:11'})
        self.assertNotContains(response, "The finish time was before the start time! Please correct this and resubmit.", status_code = 200)
        self.assertContains(response, "Saved these times for this event!", status_code = 200)   

    def test_event_time_add_date_end(self):
        ''' Adding an EventTime with date end time'''
        Membership.objects.create(person = self.fergus, group = self.group, assoctype = "CTL")
        response = self.client.post(reverse('Events.views.NowAddTime', args=[self.event.id]), {
            'start': '2008-12-31 10:20:11',
            'finish': '2009-01-01'})
        self.assertNotContains(response, "The finish time was before the start time! Please correct this and resubmit.", status_code = 200)
        self.assertContains(response, "Saved these times for this event!", status_code = 200)   

    def test_event_time_add_date_start(self):
        ''' Adding an EventTime with date start time only'''
        Membership.objects.create(person = self.fergus, group = self.group, assoctype = "CTL")
        response = self.client.post(reverse('Events.views.NowAddTime', args=[self.event.id]), {
            'start': '2008-12-31'})
        # self.assertEquals(response.content, "")
        self.assertNotContains(response, "The finish time was before the start time! Please correct this and resubmit.", status_code = 200)
        self.assertContains(response, "Saved these times for this event!", status_code = 200)   

    def test_set_reminder_hour(self):
        ''' Set 1 hour reminder for event '''
        response = self.client.get(reverse('Events.views.SetReminders', args=[self.event.id]), {'hour': '1'})
        self.assertRedirects(response, reverse('Events.views.ViewEvent', args=[self.event.id]), host = self.host)
        self.assertEquals(Reminder.objects.filter(person = self.fergus, event = self.event, assoctype = "EML_HR").count(), 1)
        self.assertEquals(Reminder.objects.filter(person = self.fergus, event = self.event, assoctype = "EML_DY").count(), 0)

    def test_set_reminder_day(self):
        ''' Set 1 day reminder for event '''
        response = self.client.get(reverse('Events.views.SetReminders', args=[self.event.id]), {'day': '1'})
        self.assertRedirects(response, reverse('Events.views.ViewEvent', args=[self.event.id]), host = self.host)
        self.assertEquals(Reminder.objects.filter(person = self.fergus, event = self.event, assoctype = "EML_DY").count(), 1)        
        self.assertEquals(Reminder.objects.filter(person = self.fergus, event = self.event, assoctype = "EML_HR").count(), 0)

    def test_set_reminder_unset(self):
        ''' Set both reminders, and then unset both '''
        response = self.client.get(reverse('Events.views.SetReminders', args=[self.event.id]), {'day': '1', 'hour': '1'})
        self.assertEquals(Reminder.objects.filter(person = self.fergus, event = self.event, assoctype = "EML_DY").count(), 1)        
        self.assertEquals(Reminder.objects.filter(person = self.fergus, event = self.event, assoctype = "EML_HR").count(), 1)
        response = self.client.get(reverse('Events.views.SetReminders', args=[self.event.id]), {'day': '0', 'hour': '0'})
        self.assertEquals(Reminder.objects.filter(person = self.fergus, event = self.event, assoctype = "EML_DY").count(), 0)        
        self.assertEquals(Reminder.objects.filter(person = self.fergus, event = self.event, assoctype = "EML_HR").count(), 0)    