from django.http import HttpResponse, HttpResponseRedirect, Http404
from Users.models import User, Membership, Reminder
from django import forms
from Events.models import Event, EventTime, EventTag, Observance
from Groups.models import Group, GroupService, ServiceProperty, GroupServiceProperty, ParseLocation
from basic.funcs import MustLogin, PidgeException, MustLoginMessage
from django.template.loader import render_to_string
from django.shortcuts import render_to_response
from django.core.exceptions import PermissionDenied
from django.core.urlresolvers import reverse
from django.conf import settings
import datetime as dt
from datetime import datetime, timedelta
from Events.forms import EventForm, AddTimeForm

def All(request):
    ''' List all events. '''
    from django.db.models import Q
    
    venue = None
    venue_gsp = None
    category = None
    loggedin = False
    allevents = None
    
    thisurl = reverse('Events.views.All')
    
    categoryid = 0
    if request.GET.get('category', False):
        try:
            categoryid = int(request.GET['category'])
        except ValueError:
            pass
            
    if 'loggedin' in request.session:
        user = User.objects.get(id = request.session['userid'])
        loggedin = True

    if categoryid == -1 or (categoryid == 0 and not loggedin):
        allevents = True
    
    events = EventTime.objects.filter(start__gte = datetime.now() - timedelta(hours = 1)).order_by('start');
            
    if loggedin and categoryid == 0:
        # Events includes any groups which the user is associated with
        # and those which fit his interests.
        
        events = events.filter(Q(event__group__membership__person = user) | Q(event__tags__user = user)).distinct()
    else:
        if categoryid > 0:
            try:
                category = EventTag.objects.get(id = request.REQUEST['category'])
                events = events.filter(event__tags__id = request.REQUEST['category'])
            except EventTag.DoesNotExist:
                pass

    search_query = ""
    if "q" in request.REQUEST:
        search_query = request.REQUEST['q']
        if len(search_query) > 0:
            events = events.filter(Q(event__title__icontains=search_query) | Q(event__description__icontains=search_query) | Q(event__group__official_name__icontains = search_query) | Q(event__group__friendly_name__icontains = search_query))
    
    if not category or not category.name == "Academic Events / Talks":
        events = events.exclude(event__group__friendly_name__exact = "talks.cam")

    tags = EventTag.objects.all().order_by('name')
    
    bc = [{'title': 'Overview'}]
    return render_to_response("events-all", {'breadcrumb': bc, 'activetab': 'Events', 'events': events, 'venue': venue, 'venuegsp': venue_gsp, 'tags': tags, 'category': category, 'allevents': allevents,'loggedin': loggedin, 'search_query': search_query, 'categoryid': categoryid, 'thisurl': thisurl})


def ViewEvent(request, eventid):
    ''' Show the details of an event. '''
    try:
        event = Event.objects.get(id=eventid)
    except(Event.DoesNotExist):
        raise Http404
    
    flags = {}
    if 'loggedin' in request.session:
        usr = User.objects.get(id = request.session['userid'])

        try:
            uea_hour = Reminder.objects.get(person = usr, event = event, assoctype='EML_HR')
            flags['email_hour'] = True
        except Reminder.DoesNotExist:
            pass
        try:
            uea_day = Reminder.objects.get(person = usr, event = event, assoctype='EML_DY')
            flags['email_day'] = True
        except Reminder.DoesNotExist:
            pass 
        
        try:
            uga = Membership.objects.get(person = usr, group=event.group, assoctype='CTL')
            flags['controller'] = True
        except Membership.DoesNotExist:
            pass
    
    location = ParseLocation(event.location_additional)
    
    eventtimes = EventTime.objects.filter(event=event).order_by('start')
    bc = [{'title': 'Overview', 'url': reverse('Events.views.All')}, {'title': event.title}]
    return render_to_response("events-event", {'breadcrumb': bc, 'activetab': 'Events', 'event': event, 'eventtimes': eventtimes, 'flags': flags, 'location_address': location['address'], 'location_allowshow': location['allowshow'], 'location_lati': location['lati'], 'location_longi': location['longi'], 'servername': settings.SERVER_NAME})


@MustLoginMessage("Please log in to add an event.")
def Add(request):
    ''' Show groups to which user can add an event / option to create group. '''
    bc = [{'title': 'Overview', 'url': reverse('Events.views.All')}, {'title': 'Add Event'}]
        
    usr = User.objects.get(id = request.session['userid'])
    ugas = Membership.objects.filter(person = usr, assoctype='CTL')
    return render_to_response("events-add", {'breadcrumb': bc, 'activetab': 'Events','ugas': ugas})

@MustLoginMessage("Please log in to add an event.")
def AddOrClone(request, groupid):
    ''' Show choice: add event or clone existing. '''
    try:
        group = Group.objects.get(id=groupid)
    except(Group.DoesNotExist):
        raise Http404

    usr = User.objects.get(id = request.session['userid'])

    try:
        uga = Membership.objects.get(person = usr, group=group, assoctype='CTL')
    except Membership.DoesNotExist:
        raise PermissionDenied
    
    events = Event.objects.filter(group = group)
    
    if events.count() == 0:
        return HttpResponseRedirect(reverse('Events.views.AddToGroup', args = [groupid]))
    
    bc = [
        {'title': 'Overview', 'url': reverse('Events.views.All')},
        {'title': 'Add Event', 'url': reverse('Events.views.Add')},
        {'title': group.official_name}  ]
    return render_to_response("events-add-or-clone", {'breadcrumb': bc, 'activetab': 'Events','events': events, 'group': group})


@MustLoginMessage("Please log in to add an event.")
def AddToGroup(request, groupid):
    ''' Add an event to a specific group. '''
    
    usr = User.objects.get(id = request.session['userid'])
    
    try:
        group = Group.objects.get(id=groupid)
    except(Group.DoesNotExist):
        raise Http404

    bc = [{'title': 'Overview', 'url': reverse('Events.views.All')},
        {'title': 'Add Event', 'url': reverse('Events.views.Add')},
        {'title': group.official_name}]

    try:
        uga = Membership.objects.get(person = usr, group=group, assoctype='CTL')
    except Membership.DoesNotExist:
        raise PermissionDenied
    
    eventClone = None
    # Does the user want to clone an existing Event for this Group?
    if 'clone' in request.REQUEST:
        try:
            eventClone = Event.objects.get(id=request.REQUEST['clone'])
        except Event.DoesNotExist:
            raise Http404
        
    if request.method == 'POST':
        myevent = Event(group = group)
        aef = EventForm(request.POST, instance = myevent)
        if aef.is_valid():
            # Save new Event and get them to add EventTimes to it
            newevent = aef.save()
            return HttpResponseRedirect(reverse('Events.views.AddTime', args=[newevent.id]))
        
    else:
        if eventClone:
            aef = EventForm(instance = eventClone)
        else:
            aef = EventForm()
    
    return render_to_response("events-add-togroup", {'breadcrumb': bc, 'activetab': 'Events','group': group, 'aef': aef})



@MustLoginMessage("Please log in to edit this event.")
def EditEvent(request, eventid):
    ''' Show and update existing event. '''
    
    try:
        event = Event.objects.get(id = eventid)
    except(Event.DoesNotExist):
        raise Http404
    
    group = event.group
    
    bc = [{'title': 'Overview', 'url': reverse('Events.views.All')},
        {'title': event.title, 'url': reverse('Events.views.ViewEvent', args=[event.id])},
        {'title':'Edit'}]
    
    usr = User.objects.get(id = request.session['userid'])
    
    try:
        uga = Membership.objects.get(person = usr, group=group, assoctype='CTL')
    except(Membership.DoesNotExist):
        raise PermissionDenied
    
    if request.method == 'POST':
        editform = EventForm(request.POST, instance = event)
        if editform.is_valid():
            editform.save()
            return HttpResponseRedirect(reverse('Events.views.ViewEvent', args=[event.id]))
    else:
        editform = EventForm(instance = event)
    return render_to_response("events-edit", {'breadcrumb': bc, 'activetab': 'Events','event': event, 'editform': editform})


@MustLoginMessage("Please log in to delete this time.")
def DeleteTime(request, eventid, eventtimeid):
    ''' Delete an EventTime for an Event. '''
    try:
        event = Event.objects.get(id = eventid)
    except Event.DoesNotExist:
        raise Http404
    
    try:
        eventtime = EventTime.objects.get(id = eventtimeid)
    except EventTime.DoesNotExist:
        raise Http404
    
    group = event.group
    
    bc = [{'title': 'Overview', 'url': reverse('Events.views.All')},
        {'title': event.title, 'url': reverse('Events.views.ViewEvent', args=[event.id])},
        {'title':'Add Time'}]
        
    usr = User.objects.get(id = request.session['userid'])
    
    try:
        uga = Membership.objects.get(person = usr, group=group, assoctype='CTL')
    except Membership.DoesNotExist:
        raise PermissionDenied

    if request.method == 'POST':
        eventtime.delete()
        return HttpResponseRedirect(reverse('Events.views.ViewEvent', args=[event.id]))
    
    return render_to_response("events-delete-time", {'breadcrumb': bc, 'activetab': 'Events','event': event, 'eventtime': eventtime})

def NowAddTime(request, eventid):
    ''' After adding an event, add times '''
    return AddTime(request, eventid, eventadded = True)

@MustLoginMessage("Please log in to add a time to this event.")
def AddTime(request, eventid, eventadded = False):
    ''' Show form to add EventTime to Event. '''
    flags = {}

    try:
        event = Event.objects.get(id=eventid)
    except(Event.DoesNotExist):
        raise Http404
    
    if eventadded:
        flags['event_added'] = True
        
    group = event.group
    
    bc = [{'title': 'Overview', 'url': reverse('Events.views.All')},
        {'title': event.title, 'url': reverse('Events.views.ViewEvent', args=[unicode(event.id)])},
        {'title':'Add Time'}]
        
    usr = User.objects.get(id = request.session['userid'])
    
    try:
        uga = Membership.objects.get(person = usr, group=group, assoctype='CTL')
    except(Membership.DoesNotExist):
        raise PermissionDenied


    if request.method == 'POST':
        myeventtime = EventTime(event = event)
        atf = AddTimeForm(request.POST, instance = myeventtime)
        if atf.is_valid():
            atf.save()
            # Now reset the form so they can add another
            atf = AddTimeForm()
            flags['added'] = True
            
    else:
        atf= AddTimeForm()

    return render_to_response("events-add-time", {'breadcrumb': bc, 'activetab': 'Events','event': event, 'atf': atf, 'MEDIA_URL': settings.MEDIA_URL, 'flags': flags})



@MustLoginMessage("You must be logged in to add an event reminder, so that we know who to email!.")
def SetReminders(request, eventid):
    ''' Update user's reminders for this Event. '''
    try:
        event = Event.objects.get(id = eventid)
    except(Event.DoesNotExist):
        raise Http404
    
    bc = [{'title': 'Overview', 'url': reverse('Events.views.All')},
        {'title': event.title, 'url': reverse('Events.views.ViewEvent', args=[event.id])},
        {'title': 'Set Reminders'}]
    
    usr = User.objects.get(id = request.session['userid'])
    
    if 'hour' in request.GET and request.GET['hour'] == '1':
        Reminder.objects.get_or_create(person = usr, event=event, assoctype='EML_HR', defaults={'assoctype': 'EML_HR'})
    else:
        try:
            Reminder.objects.get(person = usr, event = event, assoctype = 'EML_HR').delete()
        except Reminder.DoesNotExist:
            pass
    
    if 'day' in request.GET and request.GET['day'] == '1':
        Reminder.objects.get_or_create(person = usr, event=event, assoctype='EML_DY', defaults={'assoctype': 'EML_DY'})
    else:
        try:
            Reminder.objects.get(person = usr, event=event, assoctype='EML_DY').delete()
        except Reminder.DoesNotExist:
            pass
    
    return HttpResponseRedirect(reverse('Events.views.ViewEvent', args=[unicode(event.id)]))


def GetObservance(request):
    ''' Return HTML of a day's Observances. '''
    daysObservances = Observance.objects.filter(date = dt.date.today())
    responseText = ""
    for myObs in daysObservances:
        if myObs.url:
            responseText = "<a href='%s' target='_blank'>%s</a>" % (myObs.url, myObs.title)
        else:
            responseText = myObs.title
    return HttpResponse(responseText)
