'''
Groups.models
A framework for Groups [collections of Users].
'''
from django.db import models
from basic.funcs import PidgeException

class GroupCategory(models.Model):
    '''
    A type of Group.
    e.g. business, charity, society
    Maximum of one per group - thus must be mutually exclusive, and each group must fit somewhere in the list. 
    '''
    name = models.CharField(max_length=128)
    small_name = models.CharField(max_length=32)
    all_can_create = models.BooleanField()
    
    def __unicode__(self):
        return self.name

class GroupTag(models.Model):
    '''
    A category of Group.
    e.g. student services, dance, politics
    '''
    
    name = models.CharField(max_length=256)
    description = models.CharField(max_length=256, blank=True)

    def __unicode__(self):
        return self.name


class Group(models.Model):
    '''
    An entity that exists outside of, but encompasses, a group of individuals.
    '''
    
    official_name = models.CharField(
        max_length=256,
        help_text = "(required)",
        unique = True)
    
    friendly_name = models.CharField(
        "Colloquial Name",
        max_length=128,
        blank=True,
        help_text = "The name your members would use to identify the group.")
    
    description = models.TextField(
        blank=True,
        help_text="A short description of what you do.")
    
    logo_url = models.CharField(
        max_length=128,
        default="/images/genericgrouplogo.png",
        blank=True)
    
    category = models.ForeignKey(
        GroupCategory,
        verbose_name = "Type of Organisation",
        help_text='(required) Choose the category that best describes your group')
    
    tags = models.ManyToManyField(
        GroupTag,
        blank=True,
        verbose_name = "Categories",
        help_text = "Please choose all that are appropriate.")
    
    admin_name = models.CharField(
        max_length=128,
        blank=True)
    
    admin_email = models.EmailField(blank=True)
    created = models.DateTimeField(auto_now_add=True)
    last_modified = models.DateTimeField(auto_now=True)
    
    class GroupException(PidgeException): pass
    class AlreadyExists(GroupException): pass
    
    def __unicode__(self):
        return self.official_name
    
    class Meta:
        ordering = ['official_name']
        



class Service(models.Model):
    '''
    Something that a Group does.
    e.g. bar, welfare services, shop
    '''
    
    name = models.CharField(max_length=128)
    display_on_group_page = models.BooleanField(default=False)
    all_can_create = models.BooleanField(default=True)
    is_location = models.BooleanField(default=False)
    
    def __unicode__(self):
        return self.name


class GroupService(models.Model):
    '''
    Mapping of Group to Service. 
    '''
    
    group = models.ForeignKey(Group)
    service = models.ForeignKey(Service)
    active=models.BooleanField(default=True)
    
    def __unicode__(self):
        return u'%s: %s' % (self.group.official_name, self.service.name)


class ServiceProperty(models.Model):
    '''
    The information types that are associated with a Service.
    e.g. A bar has a minimum age limit. A restaurant might provide take-away.
    '''
    
    name = models.CharField(max_length=128)
    TYPE_CHOICES = (
            ('INT', 'Integer'),
            ('FPN', 'Float'),
            ('LOC', 'Location'),
            ('STR', 'String'),
            ('PRI', 'Price'),
            ('URL', 'URL'),
            ('EML', 'Email'),
    )
    type = models.CharField(max_length=3, choices=TYPE_CHOICES)
    service = models.ForeignKey(Service)
    display_on_group_page = models.BooleanField(default=False)
    service_title = models.BooleanField(default=False)
    
    def __unicode__(self):
        return u'%s %s Property' % (self.service.name, self.name)


class GroupServiceProperty(models.Model):
    '''
    Mapping between a GroupService and ServiceProperty. 
    '''
    
    groupservice = models.ForeignKey(GroupService)
    serviceproperty = models.ForeignKey(ServiceProperty)
    value = models.CharField(max_length=512)
    
    def __unicode__(self):
        return u'%s: %s Value' % (self.groupservice.group.official_name, self.groupservice.service.name)
    
    def location_address(self):
        ret = ParseLocation(self.value)
        return ret['address']
    
    def location_lati(self):
        ret = ParseLocation(self.value)
        return ret['lati']
    
    def location_longi(self):
        ret = ParseLocation(self.value)
        return ret['longi']
        
    def location_allowshow(self):
        ret = ParseLocation(self.value)
        return ret['allowshow']
    
    def GetLinkName(self):
        ''' Return name of GroupServiceProperty's web site. '''
        groupname = ""
        if (len(self.groupservice.group.friendly_name) == 0):
            groupname = self.groupservice.group.official_name
        else:
            groupname = self.groupservice.group.friendly_name
        linkname = ""
        try:
            sps = ServiceProperty.objects.get(service = self.groupservice.service, service_title=True)
            tgsp = GroupServiceProperty.objects.get(groupservice = self.groupservice.id, serviceproperty=sps.id)
            linkname = groupname + " " + tgsp.value
        except (ServiceProperty.DoesNotExist, GroupServiceProperty.DoesNotExist):
            linkname = groupname + " " + self.groupservice.service.name
        return linkname


def ParseLocation(value):
    '''
    Split apart the data contained in the location stored as a ServiceProperty.
    Returns: {address, lati, longi, allowshow}
    '''
    
    ret = {}
    ret['address']=value
    ret['lati']=''
    ret['longi']=''
    ret['allowshow']=False
    
    parts = value.split('|')
    if len(parts)==2:
        ret['address']=parts[0]
        locparts = parts[1].split(':')
        if len(locparts)==2:
            ret['lati']=locparts[0]
            ret['longi']=locparts[1]
            ret['allowshow']=True
    return ret


class MembershipInvitation(models.Model):
    '''
    Invitations to Groups sent to email addresses.  
    '''
    group = models.ForeignKey(Group)
    email = models.EmailField()
    emailsent = models.BooleanField(default=False)
    linkkey = models.CharField(max_length=40, unique=True)
    
    def __unicode__(self):
        return u'%s %s' % (self.email, self.group.official_name)
        
    class AlreadyController(PidgeException): pass
