'''
Groups.tests.models
'''

from Users.models import User, Membership
from Groups.models import GroupCategory, GroupTag, Group, Service, GroupService, ServiceProperty, GroupServiceProperty, MembershipInvitation
from Groups.forms import GroupForm
from django.core import mail
from django.test import TestCase

class GroupManagerTests(TestCase):
    
    def setUp(self):
        self.fergus = User.objects.create(firstname="Fergus", lastname="Ferrier", email="fff21@cam.ac.uk",password = "blah")
        self.fergus.UpdatePassword("password")
        self.category = GroupCategory.objects.create(name = "ThusTag", all_can_create = True)
        self.tagone = GroupTag.objects.create(name = "TagEinz", description = "Here's the first.")
        self.tagtwo = GroupTag.objects.create(name = "SPAM", description = "The tanned variety.")
        
    # Register Group

    def test_register(self):
        ''' Minimal details > returns group '''
        group = {
            'official_name': "The Pete Peart Show",
            'category': self.category.id
            }
        
        mygroupform = GroupForm(group)
        self.assert_(mygroupform.is_valid())
        mynewgroup = mygroupform.save()
        
        self.assertEquals(mynewgroup.official_name, group['official_name'])

    def test_register_all(self):
        ''' All possible details > returns group '''
        from Events.models import EventTag
        
        tags = [self.tagone, self.tagtwo]
        group = {
            'official_name': "The Pete Peart Show",
            'category': self.category.id,
            'friendly_name': "PP Show",
            'description': "Terribly exciting",
            'tags': [tag.id for tag in tags]
            }
        
        mygroupform = GroupForm(group)
        self.assert_(mygroupform.is_valid())
        mynewgroup = mygroupform.save()
        
        self.assertEquals(mynewgroup.official_name, group['official_name'])
        self.assertEquals(mynewgroup.category.id, group['category'])
        self.assertEquals(mynewgroup.friendly_name, group['friendly_name'])
        self.assertEquals(mynewgroup.description, group['description'])
        for tag in tags:
            self.assert_(tag in mynewgroup.tags.all())

    def test_register_duplicate(self):
        ''' Group with same official name as existing > AlreadyExists '''
        group = {
            'official_name': "The Pete Peart Show",
            'category': self.category.id
            }
        
        mygroupform = GroupForm(group)
        self.assert_(mygroupform.is_valid())
        mygroupform.save()
        
        mygroupform = GroupForm(group)
        self.assert_(mygroupform.errors['official_name'] == ["Group with this Official name already exists."])
        
class GroupTagTests(TestCase):
    
    def setUp(self):
        self.tagone = GroupTag.objects.create(name = "TagEinz", description = "Here's the first.")
        self.tagtwo = GroupTag.objects.create(name = "SPAM", description = "The tanned variety.")
        
    def test_create(self):
        ''' Create a GroupTag - nothing too fancy '''
        tag = {
            'name': 'My new Tag',
            'description': 'Terribly exciting'
            }
        
        mynewtag = GroupTag.objects.create(**tag)
        self.assertEquals(mynewtag.name, tag['name'])
        self.assertEquals(mynewtag.description, tag['description'])
        
    def test_unicode(self):
        ''' Check that the self description is appropriate '''
        self.assertEquals(unicode(self.tagtwo), "SPAM")

class GroupCategoryTests(TestCase):
    
    def setUp(self):
        self.category= GroupCategory.objects.create(name = "NewCat", small_name = "NC", all_can_create = True)
        
    def test_create(self):
        ''' Create a GroupCategory - nothing too fancy '''
        cate = {
            'name': 'My new Tag',
            'small_name': 'MT',
            'all_can_create': True
            }
        
        mynewcat = GroupCategory.objects.create(**cate)
        self.assertEquals(mynewcat.name, cate['name'])
        self.assertEquals(mynewcat.small_name, cate['small_name'])
        self.assertEquals(mynewcat.all_can_create, cate['all_can_create'])
        
        
    def test_unicode(self):
        ''' Check that the self description is appropriate '''
        self.assertEquals(unicode(self.category), "NewCat")
        


class ServiceTests(TestCase):
    
    def setUp(self):
        self.service = Service.objects.create(name = "Servicey")
        
    def test_unicode(self):
        ''' Check that the self description is appropriate '''
        self.assertEquals(unicode(self.service), "Servicey")
        
    def test_create(self):
        ''' Can create with all parameters '''
        service = {
            'name': 'My new Service',
            'display_on_group_page': True,
            'all_can_create': True,
            'is_location': False
            }
        mynewservice = Service.objects.create(**service)
        for item in service:
            self.assertEquals(getattr(mynewservice, item), service[item])

class GroupServiceTests(TestCase):
    
    def setUp(self):
        self.service = Service.objects.create(name = "Servicey")
        self.category = GroupCategory.objects.create(name = "NewCat", small_name = "NC", all_can_create = True)
        self.group = Group.objects.create(official_name = "My Python", category = self.category)
        self.groupservice = GroupService.objects.create(group = self.group, service = self.service)
        
        
    def test_unicode(self):
        ''' Check that the self description is appropriate '''
        self.assertEquals(unicode(self.groupservice), "My Python: Servicey")
        
    def test_create(self):
        ''' Can create with all parameters '''
        groupservice = {
            'group': self.group,
            'service': self.service,
            'active': True
            }
        mynewgroupservice = GroupService.objects.create(**groupservice)
        for item in groupservice:
            self.assertEquals(getattr(mynewgroupservice, item), groupservice[item])

class ServicePropertyTests(TestCase):
    
    def setUp(self):
        self.service = Service.objects.create(name = "Servicey")
        self.serviceprop = ServiceProperty.objects.create(name="LocationyFun", type="LOC", service=self.service, display_on_group_page = True, service_title = True)
        
    def test_unicode(self):
        ''' Check that the self description is appropriate '''
        self.assertEquals(unicode(self.serviceprop), "Servicey LocationyFun Property")

class GroupServicePropertyTests(TestCase):
    
    def setUp(self):
        self.category = GroupCategory.objects.create(name = "NewCat", small_name = "NC", all_can_create = True)
        self.group = Group.objects.create(official_name = "My Python", category = self.category)
        self.service = Service.objects.create(name = "Servicey")
        self.groupservice = GroupService.objects.create(group = self.group, service = self.service)
        self.serviceprop = ServiceProperty.objects.create(name="LocationyFun", type="LOC", service=self.service, display_on_group_page = True, service_title = True)
        self.groupserviceproperty = GroupServiceProperty.objects.create(groupservice = self.groupservice, serviceproperty = self.serviceprop, value = "It has value")
        
    def test_unicode(self):
        ''' Check that the self description is appropriate '''
        self.assertEquals(unicode(self.groupserviceproperty), "My Python: Servicey Value")
        
    

class MembershipInvitationTests(TestCase):
    
    def setUp(self):
        self.category = GroupCategory.objects.create(name = "NewCat", small_name = "NC", all_can_create = True)
        self.group = Group.objects.create(official_name = "My Python", category = self.category)
        self.invitation = MembershipInvitation.objects.create(group = self.group, email = "fff21@cam.ac.uk", emailsent = True, linkkey = "uniquey")
        
        
    def test_unicode(self):
        ''' MembershipInvitation self description is appropriate '''
        self.assertEquals(unicode(self.invitation), "fff21@cam.ac.uk My Python")
