'''
Groups.tests.views
'''
from Groups.models import Group, Service, GroupCategory, GroupTag, MembershipInvitation
from Users.models import User, Membership, MembershipClaim
from Events.models import Event, EventTime, EventTag
from django.test import TestCase, Client
from django.core import mail
from django.core.urlresolvers import reverse
from datetime import datetime, timedelta

class BasicViewTests(TestCase):
    def setUp(self):
        # Reqd so that PREPEND_WWW in settings doesn't cause all view tests to fail
        self.host = "www.testhost" 
        self.client = Client(HTTP_HOST = self.host)
        
        self.service = Service.objects.create(name = "Servicey")
        self.category = GroupCategory.objects.create(name = "NewCat", small_name = "NC", all_can_create = True)
        self.tagone = GroupTag.objects.create(name = "TagEinz", description = "Here's the first.")
        self.tagtwo = GroupTag.objects.create(name = "SPAM", description = "The tanned variety.")
        self.group = Group.objects.create(official_name = "My Python", category = self.category)
        self.group.tags.add(self.tagone)
        self.fergus = User.objects.create(firstname="Fergus", lastname="Ferrier", email="fff21@cam.ac.uk", password = "blah")
        self.fergus.UpdatePassword("password")
        self.fergus.GiveDefaults()
        self.fergusmember = Membership.objects.create(person = self.fergus, group = self.group, assoctype="MEM")
        
        self.eventtag = EventTag.objects.create(name = "Fun Things")
        
        self.event = Event.objects.create(group = self.group, title = "Dull Old Event", description = "The best thing to have happened recently.")   
        self.eventtime = EventTime.objects.create(event = self.event, start = "2007-10-30 16:00:00", finish = "2007-10-30 18:00:00")
        
        self.eventfuture = Event.objects.create(group = self.group, title = "Exciting Upcoming Event", description = "The best thing to have happened recently.")   
        self.eventfuture.tags.add(self.eventtag)
        self.eventfuturetime = EventTime.objects.create(event = self.eventfuture, start = "2009-12-30 16:00:00", finish = "2009-12-30 18:00:00")

        # Create a college
        self.collegecat, created = GroupCategory.objects.get_or_create(name = "College", small_name = "COL", defaults = {'all_can_create': False})
        self.college = Group.objects.create(official_name = "Selwine College", category = self.collegecat)

        # Event with no end time
        self.event_moment = Event.objects.create(
            group = self.group,
            title = "Momentary Event",
            description = "There is no end time.")   
        self.event_moment.tags.add(self.eventtag)
        self.event_moment_time = EventTime.objects.create(
            event = self.event_moment,
            start = "2009-12-30 16:00:00")
            
        # Event 15 minutes ago
        self.event_just_past = Event.objects.create(
            group = self.group,
            title = "Just Past Event",
            description = "Gosh you've JUST missed it.")   
        self.event_just_past.tags.add(self.eventtag)
        self.event_just_past_time = EventTime.objects.create(
            event = self.event_just_past,
            start = datetime.now() - timedelta(minutes = 15),
            finish = datetime.now() + timedelta(hours = 1))

    
    # Overview page
    
    def test_overview(self):
        ''' Not logged in > 200, message about signing in '''
        response = self.client.get(reverse('Groups.views.Overview'))
        self.assertContains(response, "You must log in to associate yourself with groups.", count=1, status_code=200)
        
    def test_overview_logged(self):
        ''' Logged in > 200, context with groups '''
        response = self.client.post(reverse('Users.views.Login', args=[""]), {'email': self.fergus.email, 'password': 'password'})
        response = self.client.get(reverse('Groups.views.Overview'))
        self.assertContains(response, "My Python", count=1, status_code=200)
        self.assertContains(response,
            "Associate yourself with other groups by using the two options on the right.",
            count=1,
            status_code=200)
    
    # All Groups
    
    def test_groups(self):
        ''' get > 200 '''
        response = self.client.get(reverse('Groups.views.All'))
        self.assertContains(response, "Selwine College", count=1, status_code=200)

    def test_groups_category(self):
        ''' get with category > 200 '''
        response = self.client.get(reverse('Groups.views.All'), {'category': self.tagone.id})
        self.assertNotContains(response, "Selwine College", status_code=200)
        self.assertContains(response, "My Python", status_code=200)

    def test_groups_query(self):
        ''' get with query > 200 + Relevant results '''
        response = self.client.get(reverse('Groups.views.All'), {'q': "college"})
        self.assertContains(response, "Selwine College", count=1, status_code=200)
        self.assertNotContains(response, "My Python", status_code=200)        
    
    # View Group
    def test_group(self):
        ''' Get existing group > 200 '''
        response = self.client.get(reverse('Groups.views.ViewGroup', args=[self.group.id]))
        self.assertContains(response, "My Python", status_code=200)
        self.assertContains(response, "Momentary Event", status_code = 200)
        self.assertContains(response, "Just Past Event", status_code = 200)
    
    def test_group_false(self):
        ''' GID not exist > 404 '''
        response = self.client.get(reverse('Groups.views.ViewGroup', args=[995756463]))
        self.assertEquals(response.status_code, 404)
    
    def test_group_showpast_off(self):
        ''' View a group without requesting past events '''
        response = self.client.get(reverse('Groups.views.ViewGroup', args=[self.group.id]))
        self.assertNotContains(response, self.event.title)
        self.assertContains(response, self.eventfuture.title)
        response = self.client.get(reverse('Groups.views.ViewGroup', args=[self.group.id]), {'showpast': '0'})
        self.assertNotContains(response, self.event.title)
        self.assertContains(response, self.eventfuture.title)

    
    def test_group_showpast(self):
        ''' View a group, and show the past events '''
        response = self.client.get(reverse('Groups.views.ViewGroup', args=[self.group.id]), {'showpast': '1'})
        self.assertContains(response, self.event.title)
        self.assertContains(response, self.eventfuture.title)

        
        
    # TODO Check hidepast functionality
    
    
    # TODO Show events that have no times
    # TODO Show events for the group in order
    # TODO Toggling of member / interested state
    # TODO Controller shows link to management interface

    # View Invitation
    def test_view_invite_bad(self):
        ''' Invitation Key does not exist '''
        response = self.client.get(reverse('Groups.views.ViewInvitation', args=["invitebad"]))
        self.assertContains(response, "This invitation appears to have been cancelled.", count=1, status_code=200)
    
    def test_view_invite(self):
        ''' Group controller Invitation Key exists '''
        self.inviteone = MembershipInvitation.objects.create(group = self.group, email = "fff21@cam.ac.uk", emailsent = True, linkkey = "yeah")
        response = self.client.get(reverse('Groups.views.ViewInvitation', args=["yeah"]))
        self.assertContains(response, "You have been invited to become a controller of the <b>My Python</b> group", count=1, status_code=200)
        
    # Reject Invitation
    def test_reject_invite(self):
        ''' Reject group control invitation. Linkkey exists. '''
        self.inviteone = MembershipInvitation.objects.create(group = self.group, email = "fff21@cam.ac.uk", emailsent = True, linkkey = "yeah")
        response = self.client.get(reverse('Groups.views.RejectInvitation', args=["yeah"]))
        self.assertContains(response, "You have successfully rejected this invitation.", count=1, status_code=200)
        self.assertEquals(MembershipInvitation.objects.filter(linkkey = "yeah").count(), 0)
    
    def test_reject_invite_bad(self):
        ''' Reject group control invitation. Linkkey does not exist. '''
        response = self.client.get(reverse('Groups.views.RejectInvitation', args=["yeah"]))
        self.assertContains(response, "This invitation appears to have been cancelled.", count=1, status_code=200)

class UserViewTests(TestCase):
    def setUp(self):
        # Reqd so that PREPEND_WWW in settings doesn't cause all view tests to fail
        self.host = "www.testhost" 
        self.client = Client(HTTP_HOST = self.host)
        
        self.service = Service.objects.create(name = "Servicey")
        self.category = GroupCategory.objects.create(name = "NewCat", small_name = "NC", all_can_create = True)
        self.tagone = GroupTag.objects.create(name = "TagEinz", description = "Here's the first.")
        self.tagtwo = GroupTag.objects.create(name = "SPAM", description = "The tanned variety.")
        self.group = Group.objects.create(official_name = "My Python", category = self.category)
        self.group.tags.add(self.tagone)
        self.fergus = User.objects.create(firstname="Fergus", lastname="Ferrier", email="fff21@cam.ac.uk", password = "blah")
        self.fergus.UpdatePassword("password")
        self.fergus.GiveDefaults()
        self.fergusmember = Membership.objects.create(person = self.fergus, group = self.group, assoctype="MEM")
        self.client.post(reverse('Users.views.Login', args=[""]), {'email': self.fergus.email, 'password': "password"})

    # Registration
    
    def test_register(self):
        ''' Page renders '''
        response = self.client.get(reverse('Groups.views.RegisterGroup'))
        self.assertEquals(response.status_code, 200)
        
    def test_register_duplicate(self):
        ''' Duplicate official_name > message on page '''
        response = self.client.post(reverse('Groups.views.RegisterGroup'), {'official_name': 'My Python', 'category': self.category.id, 'tags': (self.tagone.id,)})
        self.assertContains(response, "Group with this Official name already exists.", count=1, status_code=200)
    
    def test_register_okay(self):
        ''' Good submission > 302 to /Group/Register '''
        response = self.client.post(reverse('Groups.views.RegisterGroup'), {'official_name': 'ThePidge', 'category': self.category.id, 'tags': (self.tagone.id,)})
        newgroup = Group.objects.get(official_name = 'ThePidge')
        self.assertRedirects(response, reverse('Groups.views.RegisterGroupSuccess', args=[newgroup.id]), host = self.host)

    def test_register_invalid(self):
        ''' Invalid form renders okay '''
        response = self.client.post(reverse('Groups.views.RegisterGroup'), {'official_name': 'Myuuuu Python'})
        self.assertEquals(response.status_code, 200)
    
    
    # Registration Group Success Page
    def test_register_success(self):
        ''' Renders okay '''
        response = self.client.get(reverse('Groups.views.RegisterGroupSuccess', args=[self.group.id]))
        self.assertEquals(response.status_code, 200)

    # Accept Invitation
    def test_accept_invite(self):
        ''' Accepting a group control invitation. '''
        self.inviteone = MembershipInvitation.objects.create(group = self.group, email = "fff21@cam.ac.uk", emailsent = True, linkkey = "yeah")
        response = self.client.get(reverse('Groups.views.AcceptInvitation', args=["yeah"]))
        self.assertContains(response, "You have successfully accepted the invitation to control the My Python group.", count=1, status_code=200)
        self.assertEquals(MembershipInvitation.objects.filter(linkkey = "yeah").count(), 0)
        self.assertEquals(Membership.objects.filter(person = self.fergus, group = self.group, assoctype="CTL").count(), 1)
        
    def test_accept_invite_duplicate(self):
        ''' Accepting a group control invitation but already a controller. '''
        self.inviteone = MembershipInvitation.objects.create(group = self.group, email = "fff21@cam.ac.uk", emailsent = True, linkkey = "yeah")
        self.memship = Membership.objects.create(person = self.fergus, group = self.group, assoctype = "CTL")
        response = self.client.get(reverse('Groups.views.AcceptInvitation', args=["yeah"]))
        self.assertContains(response, "You are already a controller of the My Python group.", count=1, status_code=200)
        self.assertEquals(MembershipInvitation.objects.filter(linkkey = "yeah").count(), 1)
        self.assertEquals(Membership.objects.filter(person = self.fergus, group = self.group, assoctype="CTL").count(), 1)
        
    def test_accept_invite_bad(self):
        ''' Accepting a group control invitation that does not exist. '''
        response = self.client.get(reverse('Groups.views.AcceptInvitation', args=["noexist"]))
        self.assertContains(response, "This invitation appears to have been cancelled.", count=1, status_code=200)
        self.assertEquals(Membership.objects.filter(person = self.fergus, group = self.group, assoctype="CTL").count(), 0)
        
    
    # Claim Control
    def test_claim_control(self):
        ''' View group claim control page. '''
        response = self.client.get(reverse('Groups.views.ClaimControl', args=[self.group.id]))
        self.assertContains(response, "You must provide a brief justification as to why you should be a controller of this group.", count=1, status_code=200)
    
    def test_claim_control_bad(self):
        ''' View non-existant group's claim control page. '''
        response = self.client.get(reverse('Groups.views.ClaimControl', args=[4444444]))
        self.assertEquals(response.status_code, 404)
        
    def test_claim_control_already(self):
        ''' Claim control when already a controller of a group. '''
        assoc = Membership.objects.create(person = self.fergus, group = self.group, assoctype = "CTL")
        response = self.client.get(reverse('Groups.views.ClaimControl', args=[self.group.id]))
        self.assertRedirects(response, reverse('Groups.views.Manage', args=[self.group.id]), host = self.host)

    def test_claim_control_send(self):
        ''' Submit group control claim. '''
        response = self.client.post(reverse('Groups.views.ClaimControl', args=[self.group.id]), {'reason': 'I am like the president, innit.'})
        self.assertContains(response, "Your claim has been sent.", count=1, status_code=200)
        self.assertEquals(MembershipClaim.objects.filter(user = self.fergus, group = self.group).count(), 1)
        self.assertEquals(len(mail.outbox), 1)
    
    def test_claim_control_again(self):
        ''' Submit group control claim when user has done so before. '''
        response = self.client.post(reverse('Groups.views.ClaimControl', args=[self.group.id]), {'reason': 'I am like the committee member, innit.'})
        response = self.client.post(reverse('Groups.views.ClaimControl', args=[self.group.id]), {'reason': 'I am like the president, innit.'})
        self.assertContains(response, "Your claim has been sent.", count=1, status_code=200)
        self.assertEquals(MembershipClaim.objects.filter(user = self.fergus, group = self.group).count(), 1)
        self.assertEquals(len(mail.outbox), 1)
