'''
Groups.tests.viewsmanage
'''
from Groups.models import Group, Service, GroupCategory, GroupTag, MembershipInvitation
from Users.models import User, Membership, MembershipClaim
from django.test import TestCase, Client
from django.core import mail
from django.core.urlresolvers import reverse


class ManageViewTests(TestCase):
    def setUp(self):
        # Reqd so that PREPEND_WWW in settings doesn't cause all view tests to fail
        self.host = "www.testhost" 
        self.client = Client(HTTP_HOST = self.host)
        
        self.service = Service.objects.create(name = "Servicey")
        self.category = GroupCategory.objects.create(
            name = "NewCat",
            small_name = "NC",
            all_can_create = True)
        
        self.tagone = GroupTag.objects.create(
            name = "TagEinz",
            description = "Here's the first.")
        self.tagtwo = GroupTag.objects.create(
            name = "SPAM",
            description = "The tanned variety.")
        
        self.group = Group.objects.create(
            official_name = "My Python",
            category = self.category)
        self.group.tags.add(self.tagone)
        
        self.fergus = User.objects.create(
            firstname="Fergus",
            lastname="Ferrier",
            email="fff21@cam.ac.uk",
            password = "blah")
        self.fergus.UpdatePassword("password")
        self.fergus.GiveDefaults()
        self.fergusmember = Membership.objects.create(
            person = self.fergus,
            group = self.group,
            assoctype="CTL")
        
        self.client.post(
            reverse('Users.views.Login', args=[""]),
            {'email': self.fergus.email, 'password': "password"})

    # Manage
    
    def test_manage_nonexistent(self):
        ''' Attempt to manage a non-existent group '''
        response = self.client.get(reverse('Groups.views.Manage', args=[94444]))
        self.assertEquals(response.status_code, 404)
    
    def test_manage_denied(self):
        ''' Attempt to manage a group without permissions '''
        self.fergusmember.delete()
        response = self.client.get(reverse('Groups.views.Manage', args=[self.group.id]))
        self.assertEquals(response.status_code, 403)
    
    def test_manage(self):
        ''' Manage a group. '''
        response = self.client.get(reverse('Groups.views.Manage', args=[self.group.id]))
        self.assertEquals(response.status_code, 200)
    
    def test_manage_invite(self):
        ''' Inviting a user by email to become group controller. '''
        response = self.client.post(
            reverse('Groups.views.Manage', args=[self.group.id]),
            {'email': 'jb222@cam.ac.uk'})
        self.assertEquals(len(mail.outbox), 1)
        self.assertContains(response,
            "The following people have been sent invitations to control this group:",
            count=1,
            status_code=200)
        self.assertContains(response,
            "Invitation sent!",
            count=1,
            status_code=200)

    def test_manage_invite_again(self):
        ''' User invited a second time to become group controller. '''
        response = self.client.post(
            reverse('Groups.views.Manage', args=[self.group.id]),
            {'email': 'jb222@cam.ac.uk'})
        
        response = self.client.post(
            reverse('Groups.views.Manage', args=[self.group.id]),
            {'email': 'jb222@cam.ac.uk'})
        self.assertEquals(len(mail.outbox), 1)
        self.assertContains(response,
            "The following people have been sent invitations to control this group:",
            count=1,
            status_code=200)
        self.assertContains(response,
            "This e-mail address has already been invited to control this group.",
            count=1,
            status_code=200)
        
    def test_manage_invite_cancel(self):
        ''' Cancelling an existing group control invitation. '''
        theinvite = MembershipInvitation.objects.create(
            group = self.group,
            email = "jb222@cam.ac.uk",
            linkkey = "heytest")
        response = self.client.get(reverse('Groups.views.CancelInvitation', args=[self.group.id, theinvite.id]))
        self.assertContains(response,
            "Invitation cancelled.",
            count=1,
            status_code=200)
        self.assertEquals(MembershipInvitation.objects.filter(linkkey="heytest").count(), 0)
        
    def test_manage_invite_cancel_bad(self):
        ''' Attempt to cancel a group control invite that does not exist. '''
        response = self.client.get(reverse('Groups.views.CancelInvitation', args=[self.group.id, 4444]))
        self.assertNotContains(response, 
            "Invitation cancelled.",
            status_code=200)

    
    # Revoke Control
    
    def test_manage_revoke(self):
        ''' Revoke the controller status of another user for a group. '''
        usertwo = User.objects.create(
            firstname="J",
            lastname="JJJJJ",
            email="JJJJJ@cam.ac.uk",
            password = "blah",)
        usertwomembership = Membership.objects.create(
            person = usertwo,
            group = self.group,
            assoctype = "CTL")
        response = self.client.get(reverse('Groups.views.RevokeControl', args=[self.group.id, usertwo.id]))
        self.assertContains(response,
            "User revoked.",
            count=1,
            status_code=200)
        self.assertEquals(Membership.objects.filter(person = usertwo, group = self.group, assoctype = "CTL").count(), 0)
        
    
    def test_manage_revoke_yourself(self):
        ''' Revoke your controller status for a group. '''
        response = self.client.get(reverse('Groups.views.RevokeControl', args=[self.group.id, self.fergus.id]))
        self.assertContains(response,
            "You cannot remove yourself!",
            count=1,
            status_code=200)
        self.assertEquals(Membership.objects.filter(person = self.fergus, group = self.group, assoctype = "CTL").count(), 1)

    def test_manage_revoke_bad(self):
        ''' Revoke controller status of a user who does not exist. '''
        response = self.client.get(reverse('Groups.views.RevokeControl', args=[self.group.id, self.fergus.id]))
        self.assertNotContains(response,
            "User revoked.",
            status_code=200)
        self.assertEquals(Membership.objects.filter(person = self.fergus, group = self.group, assoctype = "CTL").count(), 1)
    
    # Change Details
    def test_manage_change(self):
        ''' Group change details page '''
        response = self.client.get(reverse('Groups.views.ChangeDetails', args=[self.group.id]))
        self.assertEquals(response.status_code, 200)
    
    def test_manage_change_duplicate(self):
        ''' User updates group official name to same as another group '''
        self.grouptwo = Group.objects.create(
            official_name = "PIDGE",
            category = self.category)
        response = self.client.post(
            reverse('Groups.views.ChangeDetails', args=[self.group.id]),
            {'official_name': 'PIDGE', 'category':self.category.id})
        self.assertContains(response,
            "Group with this Official name already exists.",
            count=1,
            status_code=200)
        self.group = Group.objects.get(id=self.group.id)
        self.assertEquals(self.group.official_name, "My Python")
    
    def test_manage_change_update(self):
        ''' User updates group details successfully '''
        response = self.client.post(
            reverse('Groups.views.ChangeDetails', args=[self.group.id]),
            {'official_name': 'PIDGE', 'category':self.category.id})
        self.assertContains(response, "Changes saved!", count=1, status_code=200)
        self.group = Group.objects.get(id=self.group.id)
        self.assertEquals(self.group.official_name, "PIDGE")
    
    # Edit Services
    def test_manage_services(self):
        ''' Access listing of services run by the group '''
        response = self.client.get(reverse('Groups.views.EditServices', args=[self.group.id]))
        self.assertEquals(response.status_code, 200)
    
    # Add Service
    def test_manage_service_add(self):
        ''' Add a service to the group '''
        newservice = Service.objects.create(name = "Web Site", all_can_create = True)
        response = self.client.get(reverse('Groups.views.AddService', args=[self.group.id, newservice.id]))
        self.assertEquals(response.status_code, 200)
        
    #   Service id that does not exist > 404    
    
    # Edit Service
    # Delete Service
    #   can delete and then readd
    # Upload Logo
    #   page renders
    #   various tests

    
    
