from django.http import HttpResponse, HttpResponseRedirect, Http404
from django.core.exceptions import PermissionDenied
from django import forms
from django.template.loader import render_to_string
from django.shortcuts import render_to_response
from Groups.models import Group, GroupTag, GroupServiceProperty, GroupCategory, ServiceProperty, MembershipInvitation
from Events.models import Event, EventTime
from Users.models import User, Membership, MembershipClaim
from Groups.viewsmanage import Manage, CancelInvitation, ChangeDetails, EditServices, DeleteService, UploadLogo, EditService, AddService, RevokeControl
from django.core.urlresolvers import reverse
from datetime import datetime, timedelta
from django.conf import settings
from django.db.models import Q
from basic.funcs import MustLogin, MustLoginMessage
from Groups.forms import RegisterGroupForm

def Overview(request):
    ''' Show Groups summary page. '''
    
    intgroups = None
    ctlgroups = None
    memgroups = None
    
    if request.session.get('loggedin'):
        user = User.objects.get(id = request.session['userid'])
        intgroups = Group.objects.filter(membership__person = user, membership__assoctype='INT').distinct()
        ctlgroups = Group.objects.filter(membership__person = user, membership__assoctype='CTL').distinct()
        memgroups = Group.objects.filter(membership__person = user, membership__assoctype='MEM').distinct()
        
    bc = [{'title': 'Overview'}]
    return render_to_response("groups-groups", {'breadcrumb': bc, 'activetab': 'Groups', 'intgroups': intgroups, 'ctlgroups': ctlgroups, 'memgroups': memgroups, 'loggedin': request.session.get('loggedin')})


def All(request):
    ''' Show page of all Groups. '''
    
    groups = Group.objects.all().order_by("official_name")
    tags = GroupTag.objects.all().order_by("name")

    category=None
    if "category" in request.REQUEST:
        try:
            category = GroupTag.objects.get(id=request.REQUEST['category'])
            groups = groups.filter(tags__id=request.REQUEST['category'])
        except(GroupTag.DoesNotExist):
            pass # No category so just display unfiltered page

    search_query = ""
    if "q" in request.REQUEST:
        search_query = request.REQUEST['q']
        if len(search_query) > 0:
            groups = groups.filter(Q(official_name__icontains=search_query) | Q(friendly_name__icontains=search_query))

    bc = [{'title': 'Overview', 'url': reverse('Groups.views.Overview')}, {'title': 'All'}]
    return render_to_response("groups-all", {'breadcrumb': bc, 'activetab': 'Groups', 'groups': groups, 'tags': tags, 'category': category, 'search_query': search_query})



def ViewGroup(request, groupid):
    ''' Show a particular Group '''
    
    try:
        group = Group.objects.get(id=groupid)
    except Group.DoesNotExist:
        raise Http404

    gsps = GroupServiceProperty.objects.filter(groupservice__group=group, groupservice__service__display_on_group_page=True, serviceproperty__display_on_group_page=True, groupservice__active=True).order_by("groupservice")

    flags = {}
    
    if request.session.get('loggedin'):
        
        usr = User.objects.get(id = request.session['userid'])
        
        if "update" in request.GET:
            
            if "member" in request.GET:
                Membership.objects.get_or_create(person = usr, group=group, assoctype='MEM', defaults={'assoctype': 'MEM'})
            else:
                try:
                    uga = Membership.objects.get(person = usr, group=group, assoctype='MEM')
                    uga.delete()
                except Membership.DoesNotExist:
                    pass

            if "interested" in request.GET:
                Membership.objects.get_or_create(person = usr, group=group, assoctype='INT', defaults={'assoctype': 'INT'})
            else:
                try:
                    uga = Membership.objects.get(person = usr, group=group, assoctype='INT')
                    uga.delete()
                except Membership.DoesNotExist:
                    pass

        usermemberships = Membership.objects.filter(person = usr, group=group)
        
        for usermembership in usermemberships:
            if usermembership.assoctype == 'CTL':
                flags['user_controller'] = True
            if usermembership.assoctype == 'INT':
                flags['user_interested'] = True
            if usermembership.assoctype == 'EML':
                flags['user_email'] = True
            if usermembership.assoctype == 'MEM':
                flags['user_member'] = True

    if request.GET.get('showpast', False) == '1':
        eventtimes = EventTime.objects.filter(event__group=group).order_by('start')
        flags['show_past'] = True
    else:
        eventtimes = EventTime.objects.filter(event__group=group, start__gte = datetime.now() - timedelta(hours = 1)).order_by('start')

    notimeevents = Event.objects.filter(group = group, eventtime__isnull=True)
    if 'linkadded' in request.GET:
        flags['link_added'] = True

    bc = [{'title': 'Overview', 'url': reverse('Groups.views.Overview')},
        {'title': 'All', 'url': reverse('Groups.views.All')},
        {'title': group.official_name}]

    return render_to_response("groups-group", {'breadcrumb': bc, 'activetab': 'Groups', 'gsps': gsps, 'group': group, 'flags': flags, 'eventtimes': eventtimes, 'notimeevents': notimeevents})



def RegisterGroupSuccess(request, groupid):
    ''' Show confirmation of creation of a Group. '''
    
    group = Group.objects.get(id = groupid)
    bc = [{'title': 'Overview', 'url': reverse('Groups.views.Overview')},
        {'title': 'Register Success'}]
    return render_to_response("groups-register-success", {'breadcrumb': bc, 'activetab': 'Groups', 'group': group})

@MustLoginMessage("You must be a registered User to create a Group. Please log in or register.")
def RegisterGroup(request):
    ''' Show form to create a Group. '''
    
    bc = [{'title': 'Overview', 'url': reverse('Groups.views.Overview')},
        {'title': 'Register'}]

    flags = {}
    
    if request.method == 'POST':
        user = User.objects.get(id = request.session['userid'])
        grf = RegisterGroupForm(request.POST, person = user)

        if grf.is_valid():
            newgroup = grf.save()
            return HttpResponseRedirect(reverse('Groups.views.RegisterGroupSuccess', args=[newgroup.id]))
            
    else:
        grf = RegisterGroupForm()

    return render_to_response("groups-register", {'breadcrumb': bc, 'activetab': 'Groups', 'registerform': grf.as_p(), 'flags': flags})


def ViewInvitation(request, linkkey):
    ''' Show Group controller invitation. '''
    bc = [{'title': 'Overview', 'url': reverse('Groups.views.Overview')}, {'title': 'Invitation'}]
    
    try:
        invitation = MembershipInvitation.objects.get(linkkey__iexact = linkkey)
    
    except MembershipInvitation.DoesNotExist:
        return render_to_response("groups-invitation-cancelled", {'breadcrumb': bc, 'activetab': 'Groups'})

    return render_to_response("groups-invitation", {'breadcrumb': bc, 'activetab': 'Groups', 'invitation': invitation})


def RejectInvitation(request, linkkey):
    ''' Confirm rejection of Group control invitation. '''
    bc = [{'title': 'Overview', 'url': reverse('Groups.views.Overview')},
        {'title': 'Invitation'}]
        
    try:
        invitation = MembershipInvitation.objects.get(linkkey__iexact = linkkey)
        invitation.delete()
        return render_to_response("groups-invitation-rejected", {'breadcrumb': bc, 'activetab': 'Groups'})
        
    except MembershipInvitation.DoesNotExist:
        return render_to_response("groups-invitation-cancelled", {'breadcrumb': bc, 'activetab': 'Groups'})


@MustLoginMessage("You must be a registered User to accept a Group Control Invitation. Please log in or register.")
def AcceptInvitation(request, linkkey):
    ''' Confirm acceptance of Group control invitation. '''
    bc = [{'title': 'Overview', 'url': reverse('Groups.views.Overview')}, {'title': 'Invitation'}]

    usr = User.objects.get(id = request.session['userid'])

    try:
        invitation = MembershipInvitation.objects.get(linkkey__iexact = linkkey)
        group = invitation.group
        usrctl, created = Membership.objects.get_or_create(group = invitation.group, person = usr, assoctype = 'CTL')
        if not created:
            raise MembershipInvitation.AlreadyController

        invitation.delete()
        return render_to_response("groups-invitation-accepted", {'breadcrumb': bc, 'activetab': 'Groups', 'group': group})

    except MembershipInvitation.AlreadyController:
        return render_to_response("groups-invitation-na", {'breadcrumb': bc, 'activetab': 'Groups', 'group': group, 'invitation': invitation})
    
    except MembershipInvitation.DoesNotExist:
        return render_to_response("groups-invitation-cancelled", {'breadcrumb': bc, 'activetab': 'Groups'})


def GetServiceProperties(request, serviceid):
    ''' Return XML of the Service Properties of a group's service. '''
    
    try:
        serviceproperties = ServiceProperty.objects.filter(service__id=serviceid)
    
    except ServiceProperty.DoesNotExist:
        raise Http404

    response = HttpResponse(mimetype='text/xml')
    response.write(render_to_string("groups-service-properties", {'serviceproperties': serviceproperties}))
    return response


@MustLoginMessage("You must be a registered User to claim control of a group. Please log in or register.")
def ClaimControl(request, groupid):
    ''' Show form to request controllership of a Group. '''
    try:
        group = Group.objects.get(id=groupid)
    except(Group.DoesNotExist):
        raise Http404

    bc = [{'title': 'Overview', 'url': reverse('Groups.views.Overview')},
        {'title': 'All', 'url': reverse('Groups.views.All')},
        {'title': group.official_name, 'url': reverse('Groups.views.ViewGroup', args=[group.id])},
        {'title': 'Claim Control'}]

    usr = User.objects.get(id = request.session['userid'])
    
    try:
        association = Membership.objects.get(person = usr, group = group, assoctype = "CTL")
    except Membership.DoesNotExist:
        pass
    else:
        return HttpResponseRedirect(reverse('Groups.views.Manage', args=[group.id]))
    
    flags = {}
    
    if request.method == 'POST':
        ccf = ClaimControlForm(request.POST)
        
        try:
            if not ccf.is_valid():
                raise ClaimControlForm.Invalid
    
            currentclaim, created = MembershipClaim.objects.get_or_create(user = usr, group = group, defaults = {'justification': ccf.cleaned_data['reason']})
            if created:
                SendMembershipClaimEmail(usr, group, ccf.cleaned_data['reason'])
            flags['claim_sent'] = True
        
        except ClaimControlForm.Invalid:
            pass
        
    else:
        ccf = ClaimControlForm()

    return render_to_response("groups-claim-control", {'breadcrumb': bc, 'activetab': 'Groups', 'group': group, 'claimform': ccf, 'flags': flags})



def SendMembershipClaimEmail(user, group, reason):
    ''' Send email to admin to indicate a request for Group controllership. '''
    from django.template.loader import render_to_string    
    
    recipient = [settings.PIDGE_CLAIMCONTROL_EMAIL]
    sender = "%s Control Claim <%s>" % (settings.PIDGE_FRIENDLY_NAME, settings.PIDGE_CLAIMCONTROL_EMAIL)
    headers = {'Reply-To': "%s <%s>" % (unicode(user), user.email)}
    subject = "%s %s" % (unicode(user), group.official_name)

    message = render_to_string('groups-email-membership-claim', { 'user':  user, 'server_name': settings.SERVER_NAME, 'group': group, 'reason': reason})

    from basic.funcs import SendEmail
    SendEmail(subject=subject, sender=sender, message=message, to=recipient, headers=headers)


class ClaimControlForm(forms.Form):
    ''' Form to request Group controllership. '''
    reason = forms.CharField(max_length=512, label="Justification", help_text="(required)")
    
    class Invalid(Exception): pass

