'''
Groups.viewsmanage - Management interface for a Group
'''

from django.http import HttpResponse, HttpResponseRedirect, Http404
from Groups.models import Group, GroupCategory, GroupServiceProperty, Service, GroupService, ServiceProperty, MembershipInvitation
from Groups.forms import GroupForm, UploadLogoForm, InviteControllerForm, LocationPropertyForm, GenericPropertyForm
from Users.models import User, Membership
from django.core.exceptions import PermissionDenied, MultipleObjectsReturned
from django.template.loader import render_to_string
from django.shortcuts import render_to_response
from django.conf import settings
from PIL import Image
import StringIO
import sha
from basic.funcs import MustLogin, PidgeException, MustLoginMessage
from django.core.urlresolvers import reverse


@MustLogin
def ChangeDetails(request, groupid):
    cd = ChangeDetailsPage(groupid)
    return cd.DisplayPage(request)

@MustLoginMessage("You must log in to manage this group.")
def Manage(request, groupid):
    mp = ManagePage(groupid)
    return mp.DisplayPage(request)

@MustLogin
def CancelInvitation(request, groupid, invitationid):
    ci = CancelInvitationPage(groupid, invitationid)
    return ci.DisplayPage(request)

@MustLogin
def RevokeControl(request, groupid, userid):
    rc = RevokeControlPage(groupid, userid)
    return rc.DisplayPage(request)

@MustLogin
def EditServices(request, groupid):
    es = EditServicesPage(groupid)
    return es.DisplayPage(request)

@MustLogin
def DeleteService(request, groupid, groupserviceid):
    ds = DeleteServicePage(groupid, groupserviceid)
    return ds.DisplayPage(request)

@MustLogin
def EditService(request, groupid, groupserviceid):
    ds = EditServicePage(groupid, groupserviceid)
    return ds.DisplayPage(request)

@MustLogin
def AddService(request, groupid, serviceid):
    ds = AddServicePage(groupid, serviceid)
    return ds.DisplayPage(request)

@MustLogin
def UploadLogo(request, groupid):
    ul = UploadLogoPage(groupid)
    return ul.DisplayPage(request)


class ManagePage(object):
    '''
    Displays the Manage/ page. Inherit from it and override the functions you want to change.
    '''
    
    flags = {}
    
    def __init__(self, groupid):
        ''' Check the group exists '''
        self.flags.clear()
        try:
            self.group = Group.objects.get(id = groupid)
        except Group.DoesNotExist:
            raise Http404

    def DisplayPage(self, request):
        ''' Construct breadcrumb and return response. '''
        self.request = request
        self.GetBreadcrumb()
        self.CheckUserGroupAccess()
        self.PageAction()
        return self.CreatePageContent()

    def PageAction(self):
        ''' After user and group checks but before page display. '''
        pass

    def CreatePageContent(self):
        ''' Generate response object for the page. '''
        
        if self.request.method == 'POST':
            inviteform = InviteControllerForm(self.request.POST)
            
            try:
                if not inviteform.is_valid():
                    raise InviteControllerForm.Invalid
                checkinvite = MembershipInvitation.objects.get(group = self.group, email__iexact = inviteform.cleaned_data['email'])
                self.flags['already_invited'] = True
            
            except InviteControllerForm.Invalid:
                pass
            
            except MembershipInvitation.DoesNotExist:
                import string
                from random import Random
                
                # Linkkey should be unique each time created, so people can't create them themselves
                # and it prevents there being duplicates in the DB. 
                linkkey = ''.join(Random().sample(string.letters+string.digits, 12))
                ci = MembershipInvitation(group = self.group, email = inviteform.cleaned_data['email'], linkkey=linkkey)
                ci.save()
                
                SendInvitationEmail(inviteform.cleaned_data['email'], self.group, self.user, linkkey)
                self.flags['invitation_sent'] = True
                
        else:
            inviteform = InviteControllerForm()

        controllers = User.objects.filter(membership__group=self.group, membership__assoctype='CTL')
        invitations = MembershipInvitation.objects.filter(group=self.group)
        return render_to_response("groups-manage", {'breadcrumb': self.breadcrumb, 'activetab': 'Groups', 'group': self.group, 'controllers': controllers, 'user': self.user, 'inviteform': inviteform, 'flags': self.flags, 'invitations': invitations})


    def GetBreadcrumb(self):
        ''' Set page breadcrumb. '''
        self.breadcrumb = [{'title': 'Overview', 'url': reverse('Groups.views.Overview')},
            {'title': 'All', 'url': reverse('Groups.views.All')},
            {'title': self.group.official_name, 'url': reverse('Groups.views.ViewGroup', args=[str(self.group.id)])},
            {'title': 'Manage'}]

    def CheckUserGroupAccess(self):
        ''' Check the user has permissions on the group. '''
        self.user = User.objects.get(id = self.request.session['userid'])
        try:
            uga = Membership.objects.get(group=self.group, person = self.user, assoctype='CTL')
        except Membership.DoesNotExist:
            raise PermissionDenied

def SendInvitationEmail(emailaddress, group, user, linkkey):
    ''' Send a Group control claim email. '''
    from django.template.loader import render_to_string    

    recipient = [emailaddress]
    sender = "%s <%s>" % (settings.PIDGE_FRIENDLY_NAME, settings.PIDGE_EMAIL)
    subject = "%s Control Invitation" % group.official_name

    message = render_to_string('groups-email-send-invitation', { 'user':  user, 'server_name': settings.SERVER_NAME, 'group': group, 'linkkey': linkkey})
    
    from basic.funcs import SendEmail
    SendEmail(subject=subject, sender=sender, message=message, to=recipient)

class CancelInvitationPage(ManagePage):
    '''
    Delete controller invitation. Then display default manage page.
    '''
    
    def __init__(self, groupid, invitationid):
        ''' remember the invitation id ''' 
        self.invitationid = invitationid
        ManagePage.__init__(self, groupid)

    def PageAction(self):
        ''' Overrides to delete invitation '''
        try:
            invitation = MembershipInvitation.objects.get(id = self.invitationid)
            invitation.delete()
            self.flags['invitation_deleted'] = True
            
        except MembershipInvitation.DoesNotExist:
            pass


class RevokeControlPage(ManagePage):
    '''
    Delete another's controller status and show default manage page.
    '''
    
    def __init__(self, groupid, userid):
        ''' Extends to remember revoked user id. '''
        self.userid = userid
        ManagePage.__init__(self, groupid)

    def PageAction(self):
        ''' Revokes a user's controller status. '''
        try:
            theuser = User.objects.get(id=self.userid)
            if theuser == self.user:
                raise Membership.SelfRevocation
            uga = Membership.objects.get(person__id = self.userid, group__id = self.group.id, assoctype='CTL')
            uga.delete()
            self.flags['user_revoked'] = True
            # TODO - email the user who has just lost the power
        
        except Membership.SelfRevocation:
            self.flags['own_revoke'] = True
        
        except Membership.DoesNotExist:
            pass

class ChangeDetailsPage(ManagePage):
    '''
    Show form to change Group basic details
    '''
    
    def GetBreadcrumb(self):
        ''' Override to update breadcrumb. '''
        self.breadcrumb = [{'title': 'Overview', 'url': reverse('Groups.views.Overview')},
            {'title': 'All', 'url': reverse('Groups.views.All')},
            {'title': self.group.official_name, 'url': reverse('Groups.views.ViewGroup', args=[str(self.group.id)])},
            {'title': 'Manage', 'url': reverse('Groups.views.Manage', args=[str(self.group.id)])},
            {'title': 'Change Details'}]

    def CreatePageContent(self):
        ''' Save model instance data, and show form. '''
        if self.request.method == 'POST':
            
            detailsform = GroupForm(self.request.POST, instance = self.group)
            if detailsform.is_valid():
                detailsform.save()
                self.flags['saved'] = True
        else:
            detailsform = GroupForm(instance = self.group)

        return render_to_response("groups-manage-change-details", {'breadcrumb': self.breadcrumb, 'activetab': 'Groups', 'detailsform': detailsform, 'flags': self.flags})


class EditServicesPage(ManagePage):
    ''' 
    Show form to select Service to change for a Group.
    '''
    
    def GetBreadcrumb(self):
        ''' Override to update breadcrumb. '''
        self.breadcrumb = [{'title': 'Overview', 'url': reverse('Groups.views.Overview')},
            {'title': 'All', 'url': reverse('Groups.views.All')},
            {'title': self.group.official_name, 'url': reverse('Groups.views.ViewGroup', args=[str(self.group.id)])},
            {'title': 'Manage', 'url': reverse('Groups.views.Manage', args=[str(self.group.id)])},
            {'title': 'Edit Services'}]

    def CreatePageContent(self):
        ''' Save Service model data and show form. '''
        
        if self.request.method == 'POST':
            try:
                if not 'serviceid' in self.request.POST:
                    raise PidgeException

                service = Service.objects.get(id=self.request.POST['serviceid'])
                gs = GroupService(group=self.group, service=service)
                gs.save()
                numsp = self.request.POST['numberserviceprops']
                flags['no_property'] = True
                
                # Do for each property
                for indx in range(0, int(numsp)):
                    try:
                        if not (self.request.POST['prop' + str(indx)] and self.request.POST['value' + str(indx)]):
                            raise PidgeException
                        propid = int(self.request.POST['prop' + str(indx)])                    
                        sp = ServiceProperty.objects.get(id=propid)
                        gsp = GroupServiceProperty(serviceproperty=sp, groupservice=gs, value=self.request.POST['value'+str(indx)])
                        gsp.save()
                        # We've set at least one property value
                        flags['no_property'] = False
                        
                    except (ServiceProperty.DoesNotExist, PidgeException):
                        pass
            
                if flags['no_property']:
                    # If we've not set any properties, then we must be deleting
                    gs.delete()
                else:
                    flags['added_service'] = True
            
            except (Service.DoesNotExist, PidgeException):
                pass
        
        gsps = GroupServiceProperty.objects.filter(groupservice__group=self.group, groupservice__active=True).order_by('groupservice')
        services = Service.objects.filter(all_can_create=True)
        return render_to_response("groups-manage-edit-services", {'breadcrumb': self.breadcrumb, 'activetab':'Groups', 'groupserviceproperties': gsps, 'services': services, 'flags': self.flags, 'group': self.group})


class DeleteServicePage(EditServicesPage):
    '''
    Delete a Group Service. 
    '''
    
    def __init__(self, groupid, gsid):
        ''' Extend to remember Group Service id '''
        self.gsid = gsid
        EditServicesPage.__init__(self, groupid)

    def PageAction(self):
        ''' Delete the group service '''
        try:
            gs = GroupService.objects.get(id=self.gsid)
            gs.active = False
            gs.save()
        except GroupService.DoesNotExist:
            pass


class EditServicePage(ManagePage):
    ''' 
    Show form to change Service details for a Group.
    '''
    
    def __init__(self, groupid, gsid):
        ''' Extend to remember GroupService ID '''
        try:
            self.groupservice = GroupService.objects.get(id=gsid)
            self.service=self.groupservice.service
        except GroupService.DoesNotExist:
            raise Http404
        self.isAdd = False
        ManagePage.__init__(self, groupid)

    def GetBreadcrumb(self):
        ''' Override to change breadcrumb '''
        self.breadcrumb = [{'title': 'Overview', 'url': reverse('Groups.views.Overview')},
            {'title': 'All', 'url': reverse('Groups.views.All')},
            {'title': self.group.official_name, 'url': reverse('Groups.views.ViewGroup', args=[str(self.group.id)])},
            {'title': 'Manage', 'url': reverse('Groups.views.Manage', args=[str(self.group.id)])},
            {'title': 'Edit Services', 'url': reverse('Groups.views.EditServices', args=[str(self.group.id)])},
            {'title': self.groupservice.service.name}]

    def CreatePageContent(self):
        ''' Update details and show form. '''
        
        if self.request.method == 'POST':
            if self.isAdd:
                self.groupservice, created = GroupService.objects.get_or_create(group=self.group, service=self.service)
            else:
                GroupServiceProperty.objects.filter(groupservice = self.groupservice).delete()
            
            serviceproperties = ServiceProperty.objects.filter(service=self.groupservice.service).order_by('id')
            
            for serviceproperty in serviceproperties:
                
                if serviceproperty.type == "LOC":
                    f = LocationPropertyForm(self.request.POST, prefix=serviceproperty.id)
                else:
                    f = GenericPropertyForm(self.request.POST, prefix=serviceproperty.id)
                
                if f.is_valid():
                    if len(f.cleaned_data['value']) > 0:
                        newgsp = GroupServiceProperty(groupservice=self.groupservice, serviceproperty=serviceproperty, value=f.cleaned_data['value'])
                        newgsp.save()
            return HttpResponseRedirect(reverse('Groups.views.EditServices', args=[str(self.group.id)]))

        serviceproperties = ServiceProperty.objects.filter(service=self.service).order_by('id')
        form = ""
        for serviceproperty in serviceproperties:
            value=""
            if not self.isAdd:
                try:
                    gsp = GroupServiceProperty.objects.get(groupservice=self.groupservice, serviceproperty=serviceproperty)
                    value = gsp.value
                except (GroupServiceProperty.DoesNotExist, MultipleObjectsReturned):
                    pass

            if serviceproperty.type == "LOC":
                f = LocationPropertyForm({str(serviceproperty.id) + '-value': value}, prefix = serviceproperty.id)
            else:
                f = GenericPropertyForm({str(serviceproperty.id) + '-value': value}, prefix = serviceproperty.id)
            form += "<p><label>" + serviceproperty.name + ":</label> " + unicode(f['value']) + "</p>"

        return render_to_response("groups-manage-edit-service", {'breadcrumb': self.breadcrumb, 'activetab': 'Groups', 'service': self.service, 'group':self.group, 'serviceproperties': serviceproperties, 'form': form})

class AddServicePage(EditServicePage):
    '''
    Add a new Service to a Group. 
    '''

    def __init__(self, groupid, serviceid):
        ''' Extend to remember Service. '''
        ManagePage.__init__(self, groupid)
        try:
            self.service = Service.objects.get(id = serviceid)
            self.isAdd = True
        except Service.DoesNotExist:
            raise Http404

    def GetBreadcrumb(self):
        ''' Override to update breadcrumb '''
        self.breadcrumb = [
            {'title': 'Overview', 'url': reverse('Groups.views.Overview')},
            {'title': 'All', 'url': reverse('Groups.views.All')},
            {'title': self.group.official_name, 'url': reverse('Groups.views.ViewGroup', args=[str(self.group.id)])},
            {'title': 'Manage', 'url': reverse('Groups.views.Manage', args=[str(self.group.id)])},
            {'title': 'Edit Services', 'url': reverse('Groups.views.EditServices', args=[str(self.group.id)])},
            {'title': self.service.name}
            ]


class UploadLogoPage(ManagePage):
    '''
    Show form to upload a logo for a Group.
    '''
    def GetBreadcrumb(self):
        ''' Override to update breadcrumb '''
        self.breadcrumb = [
            {'title': 'Overview', 'url': reverse('Groups.views.Overview')},
            {'title': 'All', 'url': reverse('Groups.views.All')},
            {'title': self.group.official_name, 'url': reverse('Groups.views.ViewGroup', args=[str(self.group.id)])},
            {'title': 'Manage', 'url': reverse('Groups.views.Manage', args=[str(self.group.id)])},
            {'title': 'Upload Logo'}
            ]

    def CreatePageContent(self):
        ''' Upload a logo. '''

        if self.request.method == 'POST':
            uploadlogoform = UploadLogoForm(self.request.POST, self.request.FILES)
            
            try:

                if not uploadlogoform.is_valid():
                    raise UploadLogoForm.Invalid

                imageupload = self.request.FILES['image']
                img = Image.open(StringIO.StringIO(imageupload.read()))
                
                width = img.size[0]
                height = img.size[1]
                max_width = 220
                max_height = 150
                target_aspect = max_height / max_width
                aspect = float(height)/float(width)

                
                if width > max_width or height > max_height:
                    if aspect > target_aspect:
                        newheight = max_height
                        newwidth = width * (float(newheight) / float(height))
                    else:
                        newwidth = max_width
                        newheight = height * (float(newwidth) / float(width))
                    img = img.resize(
                        (float(newwidth), float(newheight)),
                        Image.ANTIALIAS)

                img.save(settings.PIDGE_GROUP_LOGO_DIR + str(self.group.id) + ".png")
                self.group.logo_url = "/images/Groups/%s.png" % str(self.group.id)
                self.group.save()
            
            except UploadLogoForm.Invalid:
                pass
            
            except IOError:
                self.flags['not_image'] = True
                
            else: 
                return render_to_response("groups-manage-upload-logo-success", {'breadcrumb': self.breadcrumb, 'activetab': 'Groups', 'group': self.group, 'uploadlogoform': uploadlogoform, 'flags': self.flags})

        else:
            uploadlogoform = UploadLogoForm()

        return render_to_response("groups-manage-upload-logo", {'breadcrumb': self.breadcrumb, 'activetab':'Groups', 'group':self.group, 'uploadlogoform': uploadlogoform, 'flags': self.flags})


