#!/usr/local/bin/python
# coding: utf-8
'''
PocketPidge.py - Produce PDF pocket diaries for users. 
'''

import datetime
from reportlab.pdfgen.canvas import Canvas
from reportlab.lib.pagesizes import A4
from reportlab.lib.styles import getSampleStyleSheet, ParagraphStyle
from reportlab.lib.enums import TA_JUSTIFY, TA_CENTER, TA_RIGHT
from reportlab.platypus import Paragraph, KeepTogether, BaseDocTemplate, PageTemplate, FrameBreak, Macro, Spacer, Image, PageBreak, Table, TableStyle
from reportlab.platypus import Frame as PlatypusFrame
from reportlab.platypus.flowables import Flowable
from reportlab.lib import colors
from PIL import Image as ImageHandler

from django.core.management import setup_environ
import settings
setup_environ(settings)

from Users.models import User, Membership
from Events.models import Observance, Event, EventTime
from Groups.models import Group

def trunc(string, maxlen):
    '''
    Return a version of the input string which is of a maximum length and ends with "..."
    '''
    if len(string) > maxlen:
        string = string[0:maxlen-3] + "..."
    return string

class Line(Flowable):
    '''
    A line flowable.
    '''
    def __init__(self, xoffset=0):
        self.xoffset = xoffset
    def wrap(self, *args):
        return (self.xoffset, 0)
    def draw(self):
        canvas = self.canv
        canvas.setLineWidth(1)
        canvas.setFillColorRGB(0, 0, 0)
        canvas.setStrokeColorRGB(0, 0, 0)
        canvas.line(5, 5, 80, 5)

class TheMap(Flowable):
    '''
    Image of Cambridge Map.
    '''
    def __init__(self, xoffset=0):
        self.xoffset = xoffset
    def wrap(self, *args):
        return (self.xoffset, 0)
    def draw(self):
        canvas = self.canv
        canvas.drawImage("centre2_8000_rotated.jpg", -10,-260, width=400,height=280)
        
# Create the styles for the document
logoStyle = ParagraphStyle('normal')
logoStyle.fontSize=16
logoStyle.leading=15
logoStyle.rightindent=5
logoStyle.backColor='grey'
#logoStyle.textColor='white'
logoStyle.borderWidth=5
logoStyle.borderColor='grey'

rightStyle = ParagraphStyle('normal')
rightStyle.alignment = TA_RIGHT
rightStyle.spaceBefore = 10

paraStyle = ParagraphStyle('normal')
paraStyle.spaceAfter = 10
paraStyle.alignment = TA_JUSTIFY

medStyle = ParagraphStyle('normal')
medStyle.fontSize = 8
medStyle.leading = 8
medStyle.spaceAfter = 10
medStyle.alignment = TA_JUSTIFY

smallStyle = ParagraphStyle('normal')
smallStyle.fontSize = 6
smallStyle.leading = 6
smallStyle.spaceAfter = 0
smallStyle.alignment = TA_JUSTIFY


centreNormalStyle = ParagraphStyle('normal')
centreNormalStyle.alignment = TA_CENTER
centreNormalStyle.spaceAfter = 10

style = getSampleStyleSheet()
headlineStyle = style["Heading2"]

centreStyle = style["Heading1"]
centreStyle.alignment = TA_CENTER

class Frame:
    '''
    A placeholder for the contents of one side of a page. 
    '''
    def __init__(self, user):
        '''Set the user for the page.'''
        self.items = []
        self.user = user
    def makeItems(self):
        '''Return the frame's Flowables.'''
        pass
    def makeFrame(self):
        '''Return a KeepTogether-ed flowable set.'''
        self.makeItems()
        return KeepTogether(self.items)

class BasicFrame(Frame):
    '''
    Frame with a heading and line. 
    '''
    heading = "Hey"
    def makeItems(self):
        '''Extend flowables with heading and line.'''
        self.items.append(Paragraph("<img src='pidge.png' width=15 height=10 valign=middle /> &nbsp; " + self.heading, headlineStyle))
        self.items.append(Line())

class FrontCover(Frame):
    '''
    Front page frame.
    '''
    def makeItems(self):
        self.items.append(Spacer(60,60))
        
        self.items.append(Paragraph("<img src='pidge.png' width=88 height=57 valign=middle /> " + settings.PIDGE_FRIENDLY_NAME, logoStyle))
        self.items.append(Paragraph(settings.PIDGE_POCKET_NAME, rightStyle))
        
        self.items.append(Spacer(30,30))
        self.items.append(Paragraph("This %s belongs to:" % settings.PIDGE_POCKET_NAME, centreNormalStyle))
        self.items.append(Paragraph(self.user.firstname + ' ' + self.user.lastname, centreStyle))
        
        try:
            colleges = Membership.objects.filter(group__category__name="College", user=self.user)
            self.college = colleges[0]
        except:
            pass
        else:
            imageSrc = "../I" + self.college.group.logo_url[2:]
            print imageSrc
            collegeCrest = ImageHandler.open(imageSrc)
            if (collegeCrest.size[0] > 25) or (collegeCrest.size[1] > 25):
                aspect = float(collegeCrest.size[1])/float(collegeCrest.size[0])
                if (aspect>1):
                    newheight = 25
                    newwidth = int(collegeCrest.size[0] * (float(newheight) / float(collegeCrest.size[1])))
                else:
                    newwidth = 25
                    newheight = int(collegeCrest.size[1] * (float(newwidth) / float(collegeCrest.size[0])))
            self.items.append(Paragraph("<img src='" + imageSrc + "' width=" + str(newwidth) + " height=" + str(newheight) + " valign=middle /> &nbsp; " + self.college.group.official_name, centreStyle))
            

class BackCover(BasicFrame):
    heading = "All Good Things..."
    
class LocationsOfInterest(BasicFrame):
    heading = "Locations of Interest"
    
class FutureAppointmentsFirst(BasicFrame):
    '''
    Future Appointments frame - front side. 
    '''
    
    heading = "Future Appointments"
    def makeItems(self):
        BasicFrame.makeItems(self)
        self.items.append(Paragraph("""
        Mon, 5th Jan 2009 - Lent Term Begins<br />
        <b>Tue, 13th Jan 2009 - Full Lent Term Begins<br />
        Fri, 13th Mar 2009 - Full Lent Term Ends</b><br />
        Wed, 25th Mar 2009 - Lent Term Ends<br /><br />
        
        Fri, 10th Apr 2009 - Easter Term Begins<br />
        <b>Tue, 21st Apr 2009 - Full Easter Term Begins<br />
        Fri, 12th Jun 2009 - Full Easter Term Ends</b><br />
        Thu, 18th Jun 2009 - Easter Term Ends
        
        """,medStyle))

class FutureAppointments(BasicFrame):
    '''
    Future Appointments frame - back side, blank.
    '''
    
    heading = "Future Appointments"
    
class CambridgeMap(BasicFrame):
    '''
    Future Appointments frame - back side, blank.
    '''
    heading = "Cambridge Map"
    def makeItems(self):
        BasicFrame.makeItems(self)
        #self.items.append(TheMap())

class UserGuide(BasicFrame):
    '''
    Introduction to the PocketPidge. 
    '''
    heading = "PocketPidge User Guide"
    def makeItems(self):
        BasicFrame.makeItems(self)
        self.items.append(Paragraph("""<b>Further Event Information</b><br />
It's impossible to print full details in, so we've designed this diary to complement the web site - visit <b>MyPidge.com/Events</b> for further details [locations, full descriptions] on any of the events printed herein. The web site also has many more events than we could print here.<br /><br />
<b>Questions? Comments!</b><br />Our goal is to connect you with Cambridge. Please help us to do that, and forward your questions, comments and ideas to <b>thePidge@myPidge.com</b>.""", paraStyle))


class CareersInfo(BasicFrame):
    '''
    Careers spam. 
    '''
    heading = "Career Opportunities"
    def makeItems(self):
        BasicFrame.makeItems(self)
        self.items.append(Paragraph(u"<b>Looking at Grad Recruiters?</b><br />Save yourself time, and request information automatically from any or all of the employers featured in <i>The Times' Top 100 Graduate Employers</i>.<br /><br />Just visit <b>Top100GraduateEmployers.co.uk</b> now. Every completed entry will be entered into a special prize draw to win £5,000, or one of 50 Nintendo DS Lites. ".encode("utf-8"), medStyle))
        self.items.append(Line())
        self.items.append(Paragraph(u"<b>Final Year Undergrad? Want Some Spare Cash?</b><br/>If you're actively seeking graduate employment I want to hear from you.<br /><br />You will be <b>well-paid</b> [~£10/hour] for an hour or two of your time filling out surveys of your opinions on graduate recruitment.<br /><br />I can involve you in several such surveys, but please email me [Fergus] soon as there are a limited number of places: <b>frf21@cam.ac.uk</b>.".encode("utf-8"), medStyle))


class UsefulInformation(BasicFrame):
    heading = "Useful Information"
    def makeItems(self):
        BasicFrame.makeItems(self)
        self.items.append(Paragraph("""
        <b>Welfare</b><br />
        Emergency Services - <b>999</b> or <b>112</b><br /><i>For use in emergency situations. 24hrs.</i><br /><br />
        Cambridgeshire Police - <b>0845 456 456 4</b><br /><i>For all non-emergencies. 24hrs. </i><br /><br />
        Samaritans - <b>08457 90 90 90</b><br /><i>Confidential listening service. 24hrs.</i><br /><br />
        NHS Direct - <b>0845 4647</b><br /><i>Medical help and advice. 24hrs.<br />Call 999 in an emergency.</i><br /><br />
        <b>Local Services</b><br />
        Panther Taxis - <b>01223 715 715</b>""", medStyle))

class ToDo(BasicFrame):
    heading = "To Do Items"
    def makeItems(self):
        BasicFrame.makeItems(self)
        self.items.append(Paragraph("Print more @ myPidge.com/Pocket", paraStyle))

class Notes(BasicFrame):
    heading = "Notes"
    def makeItems(self):
        BasicFrame.makeItems(self)
        self.items.append(Paragraph("To add more pages, just cut up A4 paper to A7-sized pieces, and hole-punch.", smallStyle))

class Groups(BasicFrame):
    heading = "Your Groups"
    def makeItems(self):
        BasicFrame.makeItems(self)
        self.items.append(Paragraph("Your groups, and the abbreviations we've used in listing events. ", paraStyle))
        userAssocs = Membership.objects.filter(user=self.user).values('group__official_name', 'group__friendly_name').order_by('group__official_name').distinct()
        for eachAssoc in userAssocs:
            # Find a suitable moniker for society
            uniqueName = ""
            if eachAssoc['group__friendly_name']:
                uniqueName = eachAssoc['group__friendly_name']
            else:
                uniqueName = eachAssoc['group__official_name']
            
            if uniqueName == "talks.cam": uniqueName = "Talk"
            
            self.items.append(Paragraph(eachAssoc['group__official_name'] + " - abbreviated as: <b>" + trunc(uniqueName,15) + "</b>", smallStyle)) 

class WeekToView(BasicFrame):
    '''
    Create a diary page. 
    '''
    heading = "Week"
    Week1Start = datetime.date(2008, 10, 9)
    daydifference = datetime.timedelta(days=1)
    

    def __init__(self, user, week, isEnd=False):
        BasicFrame.__init__(self, user)
        self.week = week
        self.startDate = self.Week1Start + self.daydifference*((week-1)*7)
        self.isEnd = isEnd
        if isEnd:
            self.startDate = self.startDate + self.daydifference*4
        if week > 8:
            self.heading = "Week 8+" + str(week-8)
        else:
            self.heading = "Week " + str(week)
    
    def getObservanceText(self, theDate):
        ''' Get day's observance '''
        observanceText = ""
        daysObservances = Observance.objects.filter(date=theDate)
        for myObs in daysObservances:
            observanceText = "<b>" + myObs.title + "</b>"
        return observanceText
        

    def makeItems(self):
        ''' Create the page. '''
        BasicFrame.makeItems(self)
        
        import random
        rangeHeight=4;
        if self.isEnd: rangeHeight=3
            
        for daysinweek in range(0, rangeHeight):
            timeinquestion = self.startDate + self.daydifference*daysinweek
            
            dayText = Paragraph("<i>%s</i>" % timeinquestion.strftime("%a, %d %b"), paraStyle)
            obsText = Paragraph("<b>%s</b>" % self.getObservanceText(timeinquestion), smallStyle)
            
            # Get events
            eventsText = []
            allEvents = EventTime.objects.filter(start__gte=timeinquestion, start__lt=(timeinquestion+self.daydifference))
            
            # Maximum of three, spilling down this priority list:
            eventSet = []
            allowedevents = 3
            
            #  1) Any events for groups they're into
            myGroups = Membership.objects.filter(user=self.user).values_list('group', flat=True).distinct()
            myGroupEvents = []
            for eachGroup in myGroups:
                group = Group.objects.get(id=eachGroup)
                groupEvents = allEvents.filter(event__group = group).values_list('event', flat=True).distinct()
                myGroupEvents.extend(groupEvents)
            
            if len(myGroupEvents) > 3:
                random.shuffle(myGroupEvents)
                eventSet.extend(myGroupEvents[0:3])
            else:
                eventSet.extend(myGroupEvents)
                
            #  2) Any events with categories they're into, except talks.cam
            #    Get full list of eventtags they like
            #    For each tag, find events and append to list
            
            #    Pop up to 3 randomly whilst checking the event ids are unique
            
            if len(eventSet) < 3:
                userTags = self.user.interests.all()
                taggedEvents = []
                
                # Aprise events not including talks.cam events
                excludedEvents = allEvents.exclude(event__group__friendly_name__exact = "talks.cam")
                
                for userTag in userTags:
                    tagEvents = excludedEvents.filter(event__tags=userTag)
                    for tagEvent in tagEvents:
                        # This does not return unique events. If an event ticks more than one of their boxes, we want a higher probability that it will show up
                        taggedEvents.append(tagEvent.event.id)
                
                random.shuffle(taggedEvents)
                for eachEvent in taggedEvents:
                    # Add it to our list if it's not already in the list
                    for item in eventSet:
                        if item == eachEvent: break
                    else:
                        eventSet.append(eachEvent)
                    if len(eventSet) == 3:
                        break
            
            
            #  3) Talks.cam stuff if they like talks, broadcastable
            #    Get talks that are labelled as broadcastable by talks.cam
            #    If too many, pop randomly
            
            
            if len(eventSet) < 3:
                if self.user.interests.filter(name__exact='Academic Events / Talks').count() == 1:
                    # and bit set to publishable
                    talkscamEvents = allEvents.filter(event__group__friendly_name__exact = "talks.cam").distinct()
                    talkSet = []
                    for talk in talkscamEvents:
                        talkSet.append(talk.event.id)
                    random.shuffle(talkSet)
                    for eachTalk in talkSet:
                        eventSet.append(eachTalk)
                        if len(eventSet) == 3: break

            # Sort based on earliest start time of each event, and remove duplicates
            allEventTimes = allEvents.filter(event__id__in=eventSet).order_by("start")
            eventSetTracker = {}
            eventSetSorted = []
            for eventTime in allEventTimes:
                currentEventID = eventTime.event.id
                if not eventSetTracker.has_key(currentEventID):
                    eventSetTracker[currentEventID] = True
                    eventSetSorted.append(currentEventID)
            
                        
            # Vary length of title based on the number of events being shown
            totalEvents = len(eventSetSorted)
            eventDescriptionLength = [0, 120, 60, 40]
            thisDescriptionLength = eventDescriptionLength[totalEvents]
            
            for eventID in eventSetSorted:
                
                event = Event.objects.get(id=eventID)
                
                # Print all times of that event on that day
                allEventTimes = allEvents.filter(event=event).order_by("start")
                eventTimes = ""
                for eachEventTime in allEventTimes:
                    if eventTimes:
                        eventTimes += " &amp; " + eachEventTime.start.strftime("%H.%M")
                    else:
                        eventTimes = eachEventTime.start.strftime("%H.%M")
                        firstEventTime = eachEventTime.start
                
                # Find a suitable moniker for society
                uniqueName = ""
                if event.group.friendly_name:
                    uniqueName = event.group.friendly_name
                else:
                    uniqueName = event.group.official_name
                
                if uniqueName == "talks.cam": uniqueName = "Talk"
                # Use abbreviated name
                eventHost = trunc(uniqueName, 15)
                eventLocation = trunc(event.location_additional.split("|")[0].title(), 20)
                eventTitle = event.title.title()
                
                if eventLocation:
                    eventText = "<b>%s</b> %s - %s - <i>%s</i>" % (eventTimes, eventHost, trunc(eventTitle, thisDescriptionLength), eventLocation)
                    
                else:    
                    eventText = "<b>%s</b> %s - %s" % (eventTimes, eventHost, trunc(eventTitle, thisDescriptionLength+20))
                
                eventsText.append(Paragraph(eventText,smallStyle))
            
            
            # Print date next to observance
            data= [[dayText, obsText]]
            t = Table(data, colWidths=[65,135], rowHeights=[10], style=None, splitByRow=1, repeatRows=0, repeatCols=0)
            t.setStyle(TableStyle([('VALIGN',(0,0),(1,0),"TOP")]))
            self.items.append(t)
            
            # Print events on right
            data= [[None, eventsText]]
            t = Table(data, colWidths=[100,100], rowHeights=[50], style=None, splitByRow=1, repeatRows=0, repeatCols=0)
            t.setStyle(TableStyle([('VALIGN',(0,0),(1,0),"TOP")]))
            self.items.append(t)
        
        if self.isEnd:
            self.items.append(Line())
 
            self.items.append(Paragraph("<i>Notes</i>", paraStyle))
            if self.week == 2:
                self.items.append(Paragraph("Looking for graduate work? Get the top employers to send you the information you want and save yourself time. Visit <b>Top100GraduateEmployers.co.uk</b> now.", medStyle))    
 
class PromoFront(BasicFrame):
    '''
    Selling PocketPidge.
    '''
    heading = "Free Pocket Diary"
    def makeItems(self):
        BasicFrame.makeItems(self)
        self.items.append(Paragraph("<br />- Events Prefilled to Your Interests<br />- Notepages for  Important Stuff <br />- Handy Maps of Town<br />- Easily Fits in Your Pocket<br />- Recyclable!<br />", paraStyle))

        self.items.append(Paragraph("<br /><img src='pidge.png' width=88 height=57 valign=middle /> MyPidge.com", logoStyle))
        self.items.append(Paragraph("PocketPidge", rightStyle))

        self.items.append(Paragraph("<br/><br />Get yours at <b>MyPidge.com/Pocket</b>", paraStyle))
        self.items.append(Paragraph("[If you have already applied, please pass this on to a friend]", paraStyle))


class PromoBack(BasicFrame):
    '''
    Selling MyPidge Homepage.
    '''

    heading = "MyPidge.com/Homepage"
    def makeItems(self):
        BasicFrame.makeItems(self)
        self.items.append(Paragraph("- <b>One-Click</b> to Your Favourite Sites<br />- Search Google, Wikipedia, And More<br />- <b>Boatie</b>? Get the River Flag Status<br />- Discover Cambridge Events/Societies<br />- Available at <b>Any Computer You Use</b>", paraStyle))

        self.items.append(Image("start2.png", width=155, height=120))

        self.items.append(Paragraph("<br/>Try it out at <b>MyPidge.com/Homepage</b>", paraStyle))


     
 
class FormFront(BasicFrame):
    '''
    Front side of paper application form for PocketPidge.
    '''

    heading = "Get Your MyPidge.com PocketPidge"
    def makeItems(self):
        BasicFrame.makeItems(self)
        self.items.append(Spacer(10,10))
        self.items.append(Paragraph("It's a <b>free personal organiser</b>, customised for you, and delivered to your college in a few days' time. ", paraStyle))
        self.items.append(Paragraph("Your details will be kept safe and only used to provide you with your <b>PocketPidge</b> and to customise <b>MyPidge.com</b> for you - both student-run non-profit services. ", paraStyle))
        self.items.append(Paragraph("<b>PLEASE COMPLETE ALL, IN BLOCK CAPITALS</b>", centreNormalStyle))
        tabledata = [
        ["FIRST NAME:", ""],
        ["SURNAME:", ""],
        ["CAMBRIDGE EMAIL:", "...........@CAM.AC.UK"],
        ["COLLEGE:", ""],
        ["SUBJECT:", ""],
        ["YEAR:", ""],
        ["BOATIE?", "YES / NO"],
        ]
        t = Table(tabledata, 2*[140], 1*25)
        t.setStyle(TableStyle([('ALIGN',(0,0),(0,6),'LEFT'),
        ('ALIGN',(1,0),(1,6),'LEFT'),
        ('VALIGN',(0,0),(1,6),'MIDDLE'),
        ]))
        self.items.append(t)
        self.items.append(Spacer(30,30))
        self.items.append(Paragraph("<b>PLEASE TURN OVER</b><br/>Just two more questions.", centreNormalStyle))

class FormBack(BasicFrame):
    '''
    Back side of paper application form for PocketPidge.
    '''

    heading = "Get Your MyPidge.com PocketPidge"
    def makeItems(self):
        self.items.append(Paragraph("<b>Tick Your Interests</b><br/>...and you'll get relevant events <u>printed into your PocketPidge for you</u>! ", paraStyle))
        tabledata = [
        ["[  ]  Theatre - Watching", "[  ]  Theatre - Being Involved In"],
        ["[  ]  Gigs / Contemporary Music", "[  ]  Classical Music"],
        ["[  ]  Christian Events", "[  ]  Careers Events"],
        ["[  ]  Cinema", "[  ]  Club Nights"],
        ["[  ]  LesBiGayTrans", "[  ]  Politics"],
        ["[  ]  Academic Events / Lectures", "[  ]  International Events"],
        ["[  ]  Sport", "[  ]  Volunteering"],
        ]
        t = Table(tabledata, 2*[140], 1*20)
        t.setStyle(TableStyle([('ALIGN',(0,0),(1,6),'LEFT'),
        ('VALIGN',(0,0),(1,6),'TOP'),
        ]))
        self.items.append(t)
        self.items.append(Spacer(20,20))
        self.items.append(Paragraph("<b>Your Groups and Societies</b><br/>Please list any societies you are a member of or interested in. ", paraStyle))
        self.items.append(Spacer(70,70))

        self.items.append(Paragraph("Would you be interested in helping to run and develop MyPidge?<br /> YES / NO", paraStyle))
        self.items.append(Paragraph("That's it - just hand the form back for your free sweet. ", paraStyle))
        


def makeMapCutOut():
    ''' Make PDF of PocketPidge's generic Map insert. '''

    posts = []
    pageHeight, pageWidth = A4
    A4_Landscape = [pageWidth, pageHeight]

    formPages = (
    CareersInfo(None),
    UsefulInformation(None),
    CareersInfo(None),
    UsefulInformation(None),
    CareersInfo(None),
    UsefulInformation(None),
    CareersInfo(None),
    UsefulInformation(None),
    TownMapLeft(None),
    TownMapRight(None),
    TownMapLeft(None),
    TownMapRight(None),
    TownMapLeft(None),
    TownMapRight(None),
    TownMapLeft(None),
    TownMapRight(None),
    )
    
    for (offset, frame) in enumerate(formPages):
        posts.append(frame.makeFrame())
        if len(formPages)==(offset+1):
            # Break page at end of a PocketPidge
            # But need to do this twice if on an odd page
            posts.append(PageBreak())
        else:
            posts.append(FrameBreak())
        
    # Build File
    document = BaseDocTemplate("map.pdf", pagesize=A4_Landscape)
    template = PageTemplate(frames=createFrames(pageWidth, pageHeight, 4, 2))
    document.addPageTemplates(template)
    document.build(posts)
    

def makePocketPidge(filename):
    ''' Make PDF of PocketPidges. '''
    
    import datetime
    tempposts = {}
    posts = []
    pageHeight, pageWidth = A4
    A4_Landscape = [pageWidth, pageHeight]
    totalusers = 4
    setsofusers = 5
    framesperuser = 24
    
    myUsers = User.objects.filter(pocketpidge__isnull=True).order_by("lastname").distinct()
    
    if len(myUsers) >= (totalusers*setsofusers):
        myUsers = myUsers[:(totalusers*setsofusers)]
    
    framesperpage = 16
    
    # Index of back slot given duplexing on short edge, and sweeping left to right, then top to bottom
    A4BackMap = {
    1: '12',
    2: '11',
    3: '10',
    4: '9',
    5: '16',
    6: '15',
    7: '14',
    8: '13'}


    
    enumerateUsers = enumerate(myUsers)
    for batch in range(0,setsofusers):
        currentuser = 0
        for subbatch in range(0,totalusers):
            try:
                (userNumber, user) = enumerateUsers.next()
            except StopIteration:
                break
            print user.__unicode__()
            startPage = (currentuser * 2) + 1
            startIndex = batch*totalusers*framesperuser
            
            userPocketPidge = (
                [WeekTimetable(user),
                Groups(user),
                FrontCover(user),
                UserGuide(user),],
                
                [WeekToView(user, 2),
                WeekToView(user, 2, True),
                WeekToView(user, 7),
                WeekToView(user, 7, True),],
                
                [WeekToView(user, 3),
                WeekToView(user, 3, True),
                WeekToView(user, 8),
                WeekToView(user, 8, True),],
                
                [WeekToView(user, 4),
                WeekToView(user, 4, True),
                WeekToView(user, 9),
                WeekToView(user, 9, True),],
                
                [WeekToView(user, 5),
                WeekToView(user, 5, True),
                FutureAppointmentsFirst(user),
                FutureAppointments(user),],

                [WeekToView(user, 6),
                WeekToView(user, 6, True),
                Notes(user),
                Notes(user)]
            )
            
            # Add to posts dictionary in the order they'll be processed
            for (offset, frames) in enumerate(userPocketPidge):
                
                # first - pfoffset*framesperpage + startindex
                first = (offset*framesperpage)+ startPage + startIndex
                tempposts[first] = frames[0]
                
                # second - pfoffset*framesperpage + A4BackMap[startindex]
                second = (offset*framesperpage) + int(A4BackMap[startPage]) + startIndex
                tempposts[second] = frames[1]
                
                # third - first + 1
                third = first + 1
                tempposts[third] = frames[2]
                
                # fourth - second - 1
                fourth = second - 1
                tempposts[fourth] = frames[3]
            
            currentuser += 1
            user.pocketpidge = datetime.datetime.now()
            user.save()
        
    for value in range(0,totalusers*framesperuser*setsofusers):
        if tempposts.has_key(value+1):
            frame = tempposts[value+1]
            posts.append(frame.makeFrame())
            posts.append(FrameBreak())
        else:
            posts.append(Paragraph("---", medStyle))
            posts.append(FrameBreak())
        
        
    
    # Build File
    document = BaseDocTemplate(filename + ".pdf", pagesize=A4_Landscape)
    template = PageTemplate(frames=createFrames(pageWidth, pageHeight, 4, 2))
    document.addPageTemplates(template)
    document.build(posts)
    
def makePromo():
    ''' Make PDF of MyPidge promotional flyer. '''
    posts = []
    pageHeight, pageWidth = A4
    A4_Landscape = [pageWidth, pageHeight]
    formPages = (
    PromoFront(None),
    PromoFront(None),
    PromoFront(None),
    PromoFront(None),
    PromoFront(None),
    PromoFront(None),
    PromoFront(None),
    PromoFront(None),
    PromoBack(None),
    PromoBack(None),
    PromoBack(None),
    PromoBack(None),
    PromoBack(None),
    PromoBack(None),
    PromoBack(None),
    PromoBack(None),
    )
    for (offset, frame) in enumerate(formPages):
        posts.append(frame.makeFrame())
        if len(formPages)==(offset+1):
            # Break page at end of a PocketPidge
            # But need to do this twice if on an odd page
            posts.append(PageBreak())
        else:
            posts.append(FrameBreak())
        
    # Build File
    document = BaseDocTemplate("promo.pdf", pagesize=A4_Landscape)
    template = PageTemplate(frames=createFrames(pageWidth, pageHeight, 4, 2))
    document.addPageTemplates(template)
    document.build(posts)

def makeForm():
    ''' Make PDF of MyPidge app form. '''
    posts = []
    pageWidth, pageHeight = A4
    formPages = (
    FormFront(None),
    FormFront(None),
    FormFront(None),
    FormFront(None),
    FormBack(None),
    FormBack(None),
    FormBack(None),
    FormBack(None),
    )
    for (offset, frame) in enumerate(formPages):
        posts.append(frame.makeFrame())
        if len(formPages)==(offset+1):
            # Break page at end of a PocketPidge
            # But need to do this twice if on an odd page
            posts.append(PageBreak())
        else:
            posts.append(FrameBreak())
        
    # Build File
    document = BaseDocTemplate("form.pdf", pagesize=A4)
    template = PageTemplate(frames=createFrames(pageWidth, pageHeight, 2, 2))
    document.addPageTemplates(template)
    document.build(posts)


def createFrames(pageWidth, pageHeight, framesAcross, framesUp, framePadding=10):
    ''' Set up the page layout. '''

    frames = []
    frameWidth = pageWidth/framesAcross
    frameHeight = pageHeight/framesUp
    
    for y in range(0, framesUp):
        for x in range(0, framesAcross):
            
            # Sweep left to right
            frameX = (x)*frameWidth
            
            # Then top to bottom
            frameY = (framesUp - y - 1)*frameHeight
            
            thisFrame = PlatypusFrame(frameX+framePadding, frameY+framePadding, frameWidth-2*framePadding, frameHeight-2*framePadding)
            frames.append(thisFrame)
    
    return frames
