'''
Publicity.tests.models
'''

from Publicity.models import CommsOfficer
from Groups.models import Group, GroupCategory
from Users.models import Person
from Email.models import EmailTemplate, EmailRecipient
from django.core import mail
from django.test import TestCase

class CommsOfficerTests(TestCase):
    
    def setUp(self):
        # Add People
        self.anna = Person.objects.create(
            firstname = "Anna",
            lastname = "Binton",
            email = "abz222@cam.ac.uk")
        self.bobo = Person.objects.create(
            firstname = "Bobo",
            lastname = "The Bear",
            email = "bobothebear@cam.ac.uk")
        self.charliferous = Person.objects.create(
            firstname = "Charliferous",
            lastname = "III",
            email = "cIII@cam.ac.uk")
        
        # College Group Type
        self.college, created = GroupCategory.objects.get_or_create(name = "College", defaults = {'small_name': "COL", 'all_can_create': False})
        # College Student Union Group Type
        self.college_union, created = GroupCategory.objects.get_or_create(name = "College Students' Union", defaults = {'small_name': "CSU", 'all_can_create': False})
        
        # Create two colleges
        self.binary = Group.objects.create(official_name = "Binary College", category = self.college)
        self.porterhouse = Group.objects.create(official_name = "Porterhouse", category = self.college)

        # Create three SUs - two notionally at one college
        self.binary_juniors = Group.objects.create(official_name = "Binary Juniors", category = self.college_union)
        self.binary_seniors = Group.objects.create(official_name = "Binary Seniors", category = self.college_union)
        self.porterhouse_six_club = Group.objects.create(official_name = "Porterhouse Six Club", category = self.college_union)
        
        # Anna is comms officer of binary juniors
        self.anna_work = CommsOfficer.objects.create(
            college = self.binary,
            body = self.binary_juniors,
            body_grad = False,
            position_name = "Comms Officer",
            position_email = "anna@binaryjs.com",
            contact = self.anna
        )
        # Bobo is secretary of binary seniors
        self.bobo_work = CommsOfficer.objects.create(
            college = self.binary,
            body = self.binary_seniors,
            body_grad = True,
            position_name = "Secretary",
            position_email = "sec@binaryseniorroom.com",
            contact = self.bobo
        )        
        # Charliferous is an officer of porterhouse six club
        self.char_work = CommsOfficer.objects.create(
            college = self.porterhouse,
            body = self.porterhouse_six_club,
            body_grad = False,
            contact = self.charliferous
        )
        
        # And an emailtemplate
        self.theemail = EmailTemplate.objects.create(
            name = "Fergus' Email",
            email_reply_to = "friverlous@gmail.com",
            email_reply_to_name = "Fergus McQueen",
            email_from = "fergus@mypidge.com",
            email_from_name = "Fergus McQueen",
            subject = "{{ person.firstname }}, this is it",
            body = "Dear {{ person.firstname }}, here's an exciting email for you."
        )


    def testCommsOfficerStringRep(self):
        ''' CommsOfficer has a suitable string representation '''
        self.failUnlessEqual(unicode(self.char_work), "Charliferous III - Porterhouse Six Club relevant officer")

    def testSendOne(self):
        ''' Send a mailing to one particular comms officer. '''
        CommsOfficer.objects.send(self.theemail, [self.char_work])
        # Queues one recipient
        self.assertEquals(EmailRecipient.objects.filter(email = self.theemail).count(), 1)
        # And it's to Charliferous to his personal email address
        self.assertEquals(EmailRecipient.objects.get(email = self.theemail).alternate_address, '')
        
    def testSendAll(self):
        ''' Send a mailing to all SU's comms officers. '''
        CommsOfficer.objects.send_all(self.theemail)
        # Queues three recipients
        self.assertEquals(EmailRecipient.objects.filter(email = self.theemail).count(), 3)
        # Each message has one of their names etc
        
    def testSendUndergrad(self):
        ''' Send a mailing to undergrad SU's comms officers. '''
        CommsOfficer.objects.send_undergrad(self.theemail)
        # Creates 2 recipients - one to Anna and one to Char
        self.assertEquals(EmailRecipient.objects.filter(email = self.theemail).count(), 2)
    
    def testSendPostgrad(self):
        ''' Send a mailing to postgrad SU's comms officers. '''
        CommsOfficer.objects.send_postgrad(self.theemail)
        # Creates 1 email message
        self.assertEquals(EmailRecipient.objects.filter(email = self.theemail).count(), 1)
        # And it's to Bobo
        self.assertEquals(EmailRecipient.objects.get(email = self.theemail).alternate_address, 'sec@binaryseniorroom.com')
        