'''
Start.tests.views
'''

from Users.models import User
from Start.models import StartLink
from django.test import TestCase, Client
from django.core import mail
from django.core.urlresolvers import reverse


class BasicViewsTestCase(TestCase):
    def setUp(self):
        # Reqd so that PREPEND_WWW in settings doesn't cause all view tests to fail
        self.host = "www.testhost" 
        self.client = Client(HTTP_HOST = self.host)
        
        self.fergus = User.objects.create(firstname="Fergus", lastname="Ferrier", email="fff21@cam.ac.uk", password="password")
        self.fergus.UpdatePassword("password")
        self.fergus.GiveDefaults()
    
    # Start Page
    
    def test_start(self):
        ''' Start Page '''
        response = self.client.get(reverse('homepage'))
        self.assertEquals(response.status_code, 200)
    
    # SPSettings XML
    
    def test_start_settings(self):
        ''' SPSettings request '''
        response = self.client.get(reverse('Start.views.StartPageSettingsXML'))
        self.assertEquals(response.status_code, 200)
        self.assert_('spsettings' in self.client.cookies)
        
    def test_start_settings_logged(self):
        ''' SPSettings request when logged in '''
        response = self.client.post(reverse('Users.views.Login', args=[""]), {'email': self.fergus.email, 'password': "password"})
        response = self.client.get(reverse('Start.views.StartPageSettingsXML'))
        self.assertEquals(response.status_code, 200)
        self.assert_('spsettings' in self.client.cookies)
        
    def test_start_settings_JS(self):
        ''' JSON settings '''
        response = self.client.get(reverse('Start.views.StartPageSettingsJS'))
        self.assertEquals(response.status_code, 200)

    def test_start_settings_JS_logged(self):
        ''' SPSettings request when logged in '''
        response = self.client.post(reverse('Users.views.Login', args=[""]), {'email': self.fergus.email, 'password': "password"})
        response = self.client.get(reverse('Start.views.StartPageSettingsJS'))
        self.assertEquals(response.status_code, 200)


class UserViewsTestCase(TestCase):
        
    def setUp(self):
        # Reqd so that PREPEND_WWW in settings doesn't cause all view tests to fail
        self.host = "www.testhost" 
        self.client = Client(HTTP_HOST = self.host)

        # Create or get the Pidgeon
        thePidge,created = User.objects.get_or_create(email = "thePidge@MyPidge.com", defaults={'firstname': "The", 'lastname': "PIDGE", 'password': "hey", 'membershipstatus':"OK", 'activationlinkkey': "here"})
        # We add a pidgelink
        pidgelink = StartLink.objects.create(url = "http://www.google.com", name = "Google", owner = thePidge, defaultlink = True)

        self.fergus = User.objects.create(firstname="Fergus", lastname="Ferrier", email="fff21@cam.ac.uk", password="password", membershipstatus="OK", activationlinkkey="imokay")
        self.fergus.UpdatePassword("password")
        self.fergus.GiveDefaults()
        response = self.client.post(reverse('Users.views.Login', args=[""]), {'email': self.fergus.email, 'password': "password"})
    
    # Customise
    # Test link adding / hiding
    
    def test_customise(self):
        ''' Simple Get '''
        response = self.client.get(reverse('Start.views.Customise'))
        self.assertEquals(response.status_code, 200)
        # There are some PidgeLinks [1 now]
        self.assert_(len(response.context[0]['pidgeLinks']))
        # There are no userlinks
        self.assertEquals(len(response.context[0]['userLinks']), 0)

    def test_customise_set_boatie(self):
        ''' Show boatie box '''
        response = self.client.post(reverse('Start.views.Customise'), {'what': 'something', 'showflagstatus': 'yes'})
        self.assertContains(response, "Saved!", count=1, status_code=200)
        self.fergus = User.objects.get(id=self.fergus.id)
        self.assertEquals(self.fergus.start_showflagstatus, True)

    def test_customise_unset_boatie(self):
        ''' Hide boatie box '''
        response = self.client.post(reverse('Start.views.Customise'), {'what': 'something'})
        self.assertContains(response, "Saved!", count=1, status_code=200)
        self.fergus = User.objects.get(id=self.fergus.id)
        self.assertEquals(self.fergus.start_showflagstatus, False)
    
    # Add Group Service Link
    
    # Add Link
    
    def test_add_link(self):
        ''' Add link page loads '''
        response = self.client.get(reverse('Start.views.AddLink'))
        self.assertEquals(response.status_code, 200)
        
    def test_add_link_post(self):
        ''' Add a link - redirect to /Start/Customise/ and +1 userlinks '''
        firstlinkcount = self.fergus.links.count()
        response = self.client.post(reverse('Start.views.AddLink'), {'name': 'Test Link', 'url': 'dodgy.com'})
        self.assertRedirects(response, reverse('Start.views.Customise'), host = self.host)
        self.fergus = User.objects.get(id=self.fergus.id)
        self.assertEquals(self.fergus.links.count(), firstlinkcount + 1)
    
    # Delete Link

    def test_delete_link(self):
        ''' Delete link reduces count by one, and redirects to customise page '''
        startcount = self.fergus.links.count()
        # Get an id of one of the user's links
        links = self.fergus.links.order_by('?')
        link = links[0]
        # Delete that link
        response = self.client.get(reverse('Start.views.DeleteLink', args=[link.id]))
        self.assertRedirects(response, reverse('Start.views.Customise'), host = self.host)
        self.assertEquals(self.fergus.links.count() + 1, startcount)
        
    def test_delete_link_dummy(self):
        ''' Dummy link id sends to customise page '''
        startcount = self.fergus.links.count()
        response = self.client.get(reverse('Start.views.DeleteLink', args=[11111164545]))
        self.assertRedirects(response, reverse('Start.views.Customise'), host = self.host)
        self.assertEquals(self.fergus.links.count(), startcount)
    
    # Get flag status
    
    def test_get_flag(self):
        ''' Returns simple text '''
        response = self.client.get(reverse('Start.views.GetFlagStatus'))
        self.assertEquals(response.status_code, 200)
        
    # Get start page events
    
    def test_get_events(self):
        ''' Returns XML with status 200 '''
        response = self.client.get(reverse('Start.views.GetStartPageEvents'))
        self.assertEquals(response.status_code, 200)
    