from django.forms import ModelForm, ValidationError
from Users.models import User, Membership, CambridgeMember, Person
from django import forms
from Groups.models import Group
from django.conf import settings

class ChangeDetailsForm(ModelForm):
    ''' Form to change user details '''
    
    class Meta:
        model = User
        fields = ('firstname', 'lastname', 'email')

class ResetPasswordForm(forms.Form):
    ''' Form to reset password. '''
    email = forms.EmailField(label="E-Mail Address", help_text="(required)")

class DoResetPasswordForm(forms.Form):
    new_password = forms.CharField(widget=forms.PasswordInput())
    new_password_again = forms.CharField(widget=forms.PasswordInput())

    def clean_new_password_again(self):
        if self.cleaned_data.get('new_password') and self.cleaned_data.get('new_password_again') and self.cleaned_data['new_password'] != self.cleaned_data['new_password_again']:
            raise ValidationError('The new passwords are not the same')
        return self.cleaned_data['new_password_again']


class ChangePasswordForm(forms.Form):
    old_password = forms.CharField(widget=forms.PasswordInput())
    new_password = forms.CharField(widget=forms.PasswordInput())
    new_password_again = forms.CharField(widget=forms.PasswordInput())

    def __init__(self, data=None, instance=None, *args, **kwargs):
        self.instance = instance
        super(ChangePasswordForm, self).__init__(data, *args, **kwargs)

    def clean_old_password(self):
        if self.cleaned_data.get('old_password'):
            try:
                self.instance.CheckPassword(self.cleaned_data['old_password'])
            except User.PasswordError:
                raise ValidationError('Please type your current password correctly.')
        return self.cleaned_data['old_password']

    def clean_new_password_again(self):
        if self.cleaned_data.get('new_password') and self.cleaned_data.get('new_password_again') and self.cleaned_data['new_password'] != self.cleaned_data['new_password_again']:
            raise ValidationError('The new passwords are not the same')
        return self.cleaned_data['new_password_again']

    def save(self):
        self.instance.UpdatePassword(self.cleaned_data['new_password'])

class CambridgeRegisterForm(ModelForm):
    ''' Form for CambridgeMember registration '''
    college = forms.ModelChoiceField(queryset = Group.objects.filter(category__small_name = 'COL'), required=False)

    def __init__(self, data=None, crsid=None, pk=None, *args, **kwargs):
        self.crsid = crsid
        self.pk = pk
        super(CambridgeRegisterForm, self).__init__(data, *args, **kwargs)
    
    def save(self, *args, **kwargs):
        
        # Get existing person to add Cambridge record to
        person = Person.objects.get(pk = self.pk)

        # Create CambridgeMember and append person data
        myCM = CambridgeMember(
            crsid = self.crsid,
            subject = self.cleaned_data['subject'],
            type = self.cleaned_data['type'],
            **person.as_dict())
        myCM.save()
        
        if self.cleaned_data.get('college'):
            Membership.objects.create(person = person, group = self.cleaned_data['college'], assoctype = "MEM")

    class Meta:
        model = CambridgeMember
        fields = ('type', 'subject')

class RegisterForm(ModelForm):
    ''' Form for User registration '''
    # Update label
    start_showflagstatus = forms.BooleanField(label="Are You A Boatie?", help_text="Tick the box and you'll see rowing events and the Cam river flag status on your home page.", required=False)
    password_one = forms.CharField(max_length=32, widget=forms.PasswordInput, help_text="(required) Choose a password for %s. You can use this if you cannot get in using Raven. Recommended that you don't use the same password as for Raven." % settings.PIDGE_FRIENDLY_NAME)
    password_again = forms.CharField(max_length=32, widget=forms.PasswordInput, label='Password Again', help_text="(required) The same password again")

    class Meta:
        model = User
        fields = ('firstname', 'lastname', 'email', 'interests')

    def __init__(self, data=None, crsid=None, *args, **kwargs):
        self.crsid = crsid
        super(RegisterForm, self).__init__(data, *args, **kwargs)

    
    def clean_password_again(self):
        if self.cleaned_data.get('password_one') and self.cleaned_data.get('password_again') and self.cleaned_data['password_one'] != self.cleaned_data['password_again']:
            raise ValidationError('The passwords are not the same.')
        return self.cleaned_data['password_again']
    
    def save(self, *args, **kwargs):
        activate = False
        
        # Must add password to user first as otherwise user won't save
        self.instance.UpdatePassword(self.cleaned_data['password_one'], commit=False)
        
        # If Person previously existed, garner their details
        try:
            theperson = Person.objects.get(email = self.cleaned_data['email'])
            self.instance.id = theperson.id
            self.instance.created = theperson.created
            self.instance.last_modified = theperson.last_modified
        except Person.DoesNotExist:
            pass
        
        # Create the user
        user = super(RegisterForm, self).save(*args, **kwargs)
        
        # Now add the m2m relationships and other bits n pieces
        
        user.membershipstatus = 'EP'
        
        if self.crsid:
            if user.email == (self.crsid + "@cam.ac.uk"):
                user.membershipstatus = 'OK'
        
        if user.membershipstatus == 'EP':
            import sha
            import time
            key = sha.new(str(time.time()) + user.email).hexdigest()
            user.activationlinkkey = key
            user.SendActivationEmail()
            activate = True
        
        user.save()
        
        return user, activate