'''
Users.models - representation of people and their many facets.

'''

from django.db import models
from django.http import HttpResponseRedirect
from urllib import quote
from django.template.loader import render_to_string
from Events.models import EventTag, Event
from django.utils.encoding import smart_str
from django.conf import settings
from Groups.models import Group
from basic.funcs import PidgeException

        
class Person(models.Model):
    '''
    A person!
    '''
    
    firstname = models.CharField(
        "first name",
        max_length = 128,
        help_text = "(required)")
    
    lastname = models.CharField(
        "last name",
        max_length = 128,
        help_text = "(required)")
    
    email = models.EmailField(
        "email address",
        unique = True,
        help_text = "(required)")  
        
    groups = models.ManyToManyField(
        Group,
        blank = True,
        through = 'Membership',
        related_name = 'group_set',
        verbose_name = "group memberships")
        
    reminders = models.ManyToManyField(
        Event,
        blank = True,
        through = 'Reminder',
        verbose_name = "events attending")
    
    created = models.DateTimeField(auto_now_add = True)
    last_modified = models.DateTimeField(auto_now = True) 
    
    class Meta:
        ordering = ['lastname']

    def __unicode__(self):
        return u'%s %s' % (self.firstname, self.lastname)
        
    def as_dict(self):
        ''' Returns all values of the object as dictionary '''
        return self.__class__.objects.filter(pk=self.pk).values()[0] 

class CambridgeMember(Person):
    ''' Persons who are members of the University '''
    STUDENT_CHOICES = (
            ('NOT', 'Not a student'),
            ('U1', 'Undergraduate - 1st Year'),
            ('U2', 'Undergraduate - 2nd Year'),
            ('U3', 'Undergraduate - 3rd Year'),
            ('U4', 'Undergraduate - 4+ Year'),
            ('MPH', 'MPhil'),
            ('PHD', 'PhD'),
            ('LLM', 'LLM'),
            ('PGR', 'Postgraduate'),
    )
    type = models.CharField(
        "academic status",
        max_length = 3,
        blank = True,
        choices = STUDENT_CHOICES,
        default = 'NOT')

    SUBJECT_CHOICES = (
            ('NO', 'No Subject'),
            ('AS', 'Anglo-Saxon, Norse and Celtic'),
            ('AN', 'Anthropology'),
            ('AR', 'Archaeology'),
            ('AA', 'Archaeology and Anthropology'),
            ('AC', 'Architecture'),
            ('AM', 'Asian and Middle Eastern Studies'),
            ('BCH', 'Biochemistry'),
            ('BT', 'Biotechnology'),
            ('CE', 'Chemical Engineering'),
            ('CHE', 'Chemistry'),
            ('CL', 'Classics'),
            ('CS', 'Computer Science'),
            ('CRI', 'Criminology'),
            ('DIV', 'Divinity'),
            ('EC', 'Economics'),
            ('ED', 'Education'),
            ('EG', 'Engineering'),
            ('EN', 'English'),
            ('FIN', 'Finance'),
            ('GEN', 'Genetics'),
            ('GE', 'Geography'),
            ('HI', 'History'),
            ('HPS', 'History and Philosophy of Science'),
            ('HA', 'History of Art'),
            ('INT', 'International Studies'),
            ('LE', 'Land Economy'),
            ('LAT', 'Latin American Studies'),
            ('LA', 'Law'),
            ('LIN', 'Linguistics'),
            ('MAN', 'Management'),
            ('MAT', 'Materials Science and Metallurgy'),
            ('MA', 'Mathematics'),
            ('ME', 'Medicine'),
            ('MM', 'Modern and Medieval Languages'),
            ('MU', 'Music'),
            ('NS', 'Natural Sciences'),
            ('NEU', 'Neuroscience'),
            ('PAT', 'Pathology'),
            ('PHI', 'Philosophy'),
            ('PHY', 'Physics'),
            ('PLA', 'Plant Sciences'),
            ('PSY', 'Psychology'),
            ('SP', 'Social and Political Sciences'),
            ('TH', 'Theology'),
            ('VS', 'Veterinary Sciences'),
            ('ZO', 'Zoology'),
    )
    subject = models.CharField(
        "academic subject",
        max_length = 3,
        blank = True,
        choices = SUBJECT_CHOICES,
        default = 'NO')
    
    crsid = models.CharField(
        "raven username",
        max_length = 10,
        unique = True,
        null = True,
        blank = True)
        
    def __unicode__(self):
        return u'%s %s' % (self.firstname, self.lastname)

class UserManager(models.Manager):
    ''' Helper functions that act at the level of the collection of Users.
    c.f. helper functions that work on a per-user basis.
    '''
    
    def Activate(self, linkkey=None):
        ''' Confirm the email address of a User. '''
        if not linkkey:
            raise User.InvalidKey
        try:
            usr = User.objects.get(activationlinkkey__exact = linkkey)
        except (User.DoesNotExist, User.MultipleObjectsReturned):
            raise User.InvalidKey
        
        if usr.membershipstatus == 'OK':
            raise User.AlreadyActive
            
        if not usr.membershipstatus == 'EP':
            raise User.ActivationNotPending
        
        usr.membershipstatus = "OK"
        usr.save()
        return usr

class User(Person):
    ''' Persons who are Pidge Users '''
    
    objects = UserManager()

    password = models.CharField(
        max_length = 40,
        blank = True,
        editable = False)
    
    STATUS_CHOICES = (
            ('OK', 'Active'),
            ('EP', 'Authentication email pending'),
            ('DA', 'Deactivated'),
    )
    membershipstatus = models.CharField(
        max_length = 3,
        choices = STATUS_CHOICES,
        default = 'OK')
    
    activationlinkkey = models.CharField(
        max_length=40,
        blank = True,
        editable = False)

    password_reset_key = models.CharField(
        max_length = 12,
        blank = True,
        editable = False)
        
    interests = models.ManyToManyField(
        EventTag,
        blank = True,
        verbose_name = "personal interests",
        help_text = "Please choose at least one, and as many that apply.")
    
    groupclaims = models.ManyToManyField(
        Group,
        blank = True,
        through = 'MembershipClaim',
        related_name = 'groupclaim_set')

    start_openlinksinnew = models.BooleanField(
        default = False,
        help_text = "Open a new window when I click links on my %s Desktop." % settings.PIDGE_FRIENDLY_NAME)
    
    start_opensearchinnew = models.BooleanField(
        default = False,
        help_text = "Open a new window when I perform a search on the %s Desktop." % settings.PIDGE_FRIENDLY_NAME)
    
    start_showflagstatus = models.BooleanField(
        default = False,
        help_text = "Show me the river Cam's rowing status on my %s Desktop." % settings.PIDGE_FRIENDLY_NAME)
    
    registered = models.DateTimeField(auto_now_add = True)
    
    pocketpidge = models.DateTimeField(
        null = True,
        editable = False,
        blank = True)
    
    activation_reminder = models.DateTimeField(
        null = True,
        editable = False,
        blank = True)
    
    class LoginException(PidgeException): pass
    class DetailsError(LoginException): pass
    class NotActivated(LoginException): pass
    class Deactivated(LoginException): pass
    
    class UserException(PidgeException): pass
    class PasswordError(UserException): pass
    
    class NoCredentials(UserException): pass
    class ImpossibleToActivate(UserException): pass
    
    class RegisterException(PidgeException): pass
    class DuplicateEmail(RegisterException): pass
    class MustActivate(RegisterException): pass
    
    class ActivationException(PidgeException): pass
    class InvalidKey(ActivationException): pass
    class AlreadyActive(ActivationException): pass
    class ActivationNotPending(ActivationException): pass

    
    def GetAllLinks(self):
        ''' All links the user created, including those not currently being shown on start page '''
        from Start.models import StartLink
        userlinks = StartLink.objects.filter(owner = self).order_by('id')
        return userlinks

    def GetLinks(self):
        ''' All links on user's start page '''
        userlinks = self.links.all()
        return userlinks
    
    def GiveDefaults(self):
        ''' Give user basic data / settings. '''
        from Start.models import StartLink
        from Groups.models import GroupServiceProperty
        
        # Add initial links
        startlinks = StartLink.objects.filter(defaultlink = True)
        for startlink in startlinks:
            self.links.add(startlink)
        
        # Add links for member's groups
        gsps = GroupServiceProperty.objects.filter(groupservice__group__membership__person = self, serviceproperty__display_on_group_page = True, serviceproperty__type='URL').distinct()
        for gsp in gsps:
            linkname = gsp.GetLinkName()
            StartLink.objects.Add(self, linkname, gsp.value)

    def save(self, *args, **kwargs):
        ''' Extends save to do some error checking '''
        if not self.password:
            raise User.NoCredentials
        if self.membershipstatus == "EP" and not self.activationlinkkey:
            raise User.ImpossibleToActivate
        super(User, self).save(*args, **kwargs) # Call the "real" save() method
    
    def Authenticate(self, password=None):
        ''' Is user able to log in? '''
        
        if password:
            try:
                self.CheckPassword(password)
            except User.PasswordError:
                raise User.DetailsError
        
        if self.membershipstatus == 'EP':
            raise User.NotActivated
        if self.membershipstatus == 'DA':
            raise User.Deactivated
    
    def CheckPassword(self, password):
        import sha
        if not password:
            raise User.PasswordError
        if not self.password == sha.new(password).hexdigest():
            raise User.PasswordError
        return True

    def SetResetKey(self):
        import string
        from random import Random
        resetkey = ''.join(Random().sample(string.letters+string.digits, 12))
        resetkey = str(resetkey)
        self.password_reset_key = resetkey
        self.save()
        return resetkey
    
    def UpdatePassword(self, password, commit=True):
        import sha
        newpasswd = sha.new(password).hexdigest()
        self.password = newpasswd
        if commit == True:
            self.save()
            
    def SendActivationEmail(self):
        ''' Send User activation email.  '''
        from django.template.loader import render_to_string

        if self.membershipstatus != "EP":
            raise User.ActivationNotPending
        
        recipient = ["\"%s\" <%s>" % (unicode(self), self.email)]
        sender = "%s <%s>" % (settings.PIDGE_FRIENDLY_NAME, settings.PIDGE_EMAIL)
        subject = "%s, Please activate your account now" % (self.firstname)
        message = render_to_string('users-email-activation', { 'user':  self, 'server_name': settings.SERVER_NAME, 'linkkey': self.activationlinkkey })

        from basic.funcs import SendEmail
        SendEmail(subject=subject, sender=sender, message=message, to=recipient)

class PersonList(models.Model):
    '''
    A set of People.
    '''
    name = models.CharField(
        "list name",
        max_length = 128,
        help_text = "(required) A simple moniker for a list of people.")
    
    people = models.ManyToManyField(Person)
    
    def __unicode__(self):
        return self.name

class Reminder(models.Model):
    '''
    Reminders about Events for Users. 
    '''
    person = models.ForeignKey(Person)
    event = models.ForeignKey(Event)
    ASSOCIATION_CHOICES = (
            ('EML_HR', 'E-Mail 1 Hour'),
            ('EML_DY', 'E-Mail 1 Day'),
    )
    assoctype = models.CharField(max_length=6, choices=ASSOCIATION_CHOICES)

    def __unicode__(self):
        return u'%s %s' % (unicode(self.person), self.assoctype)


class Membership(models.Model):
    '''
    Mapping between Persons and Groups.
    '''
    person = models.ForeignKey(Person)
    group = models.ForeignKey(Group)
    ASSOCIATION_CHOICES = (
            ('CRE', 'Creator'),
            ('INT', 'Interested in'),
            ('CTL', 'Controller'),
            ('MEM', 'Member'),
            ('EML', 'Email'),
    )
    assoctype = models.CharField(max_length=3, choices=ASSOCIATION_CHOICES)
    

    class MembershipException(PidgeException): pass
    class SelfRevocation(MembershipException): pass

    def __unicode__(self):
        return u'%s %s %s' % (self.person.firstname, self.person.lastname, self.group.official_name)
    
    
class MembershipClaim(models.Model):
    '''
    Claims for membership / control of Groups made by Users.
    '''

    user = models.ForeignKey(User)
    group = models.ForeignKey(Group)
    justification = models.CharField(max_length=512)
    
    def __unicode__(self):
        return u'%s %s %s' % (self.user.firstname, self.user.lastname, self.group.official_name)



