'''
Users.tests.models
'''

from Users.models import User, Reminder, Membership, MembershipClaim, Person, CambridgeMember
from Groups.models import Group, GroupCategory
from Start.models import StartLink
from django.core import mail
from django.test import TestCase
from Users.forms import RegisterForm, CambridgeRegisterForm
from Events.models import EventTag

class UserTests(TestCase):
    
    def setUp(self):
        import sha        
        self.fergus = User.objects.create(
            firstname="Fergus",
            lastname="Ferrier",
            email="fff21@cam.ac.uk",
            password=sha.new("password").hexdigest())
        self.usertwo = User.objects.create(firstname="E", lastname="Parkinson", email="eric.parkinson@hotmail.com", password="pleaselogmein")
        
        # Create or get the Pidgeon
        thePidge,created = User.objects.get_or_create(email = "thePidge@MyPidge.com", defaults={'firstname': "The", 'lastname': "PIDGE", 'password': "hey", 'membershipstatus':"OK", 'activationlinkkey': "here"})
        # We add a pidgelink
        pidgelink = StartLink.objects.create(url = "http://www.google.com", name = "Google", owner = thePidge, defaultlink = True)


    def testUserStringRep(self):
        ''' The string representation of a user is okay '''
        self.failUnlessEqual(unicode(self.fergus), "Fergus Ferrier")
        self.failUnlessEqual(unicode(self.usertwo), "E Parkinson")
        
    def testCheckPasswordCorrect(self):    
        ''' CheckPassword when password is good '''
        self.assertEquals(self.fergus.CheckPassword("password"), True)

    def testCheckPasswordIncorrect(self):    
        ''' CheckPassword when password is bad '''
        self.assertRaises(User.PasswordError, self.fergus.CheckPassword, "password1")

    def testCheckPasswordBlank(self):    
        ''' CheckPassword when password is blank '''
        self.assertRaises(User.PasswordError, self.fergus.CheckPassword, "")
    
    def testGetAllLinks(self):
        ''' Returns QuerySet of StartLinks '''
        from Start.models import StartLink
        self.assertEquals(len(self.fergus.GetAllLinks()), 0)
        s = StartLink(name="loogle", url="http://www.google.com", owner=self.fergus)
        s.save()
        #self.fergus.links.add(s)
        self.assertEquals(len(self.fergus.GetAllLinks()), 1)

    def testGetLinks(self):
        ''' Returns QuerySet of StartLinks '''
        from Start.models import StartLink
        self.assertEquals(len(self.fergus.GetLinks()), 0)
        s = StartLink(name="loogle", url="http://www.google.com", owner=self.fergus)
        s.save()
        self.fergus.links.add(s)
        self.assertEquals(len(self.fergus.GetLinks()), 1)

    def testGiveDefaults(self):
        ''' GiveDefaults '''
        self.assertEquals(len(self.fergus.links.all()), 0)
        self.fergus.GiveDefaults()
        self.assertEquals(len(self.fergus.links.all()) > 0, True)
        #- group url in links'''
    
    def testSaveDuplicateEmail(self):    
        ''' Try adding a duplicate email address '''
        from django.db import IntegrityError
        self.usertwo.email = "fff21@cam.ac.uk"
        self.assertRaises(IntegrityError, self.usertwo.save)

    def testSaveNoCredentials(self):    
        ''' Save a user without password or crsid '''
        newuser = User(firstname="Me", lastname="Again", email="frf20@cam.ac.uk")
        self.assertRaises(User.NoCredentials, newuser.save)

    def testSaveImpossibleToActivate(self):    
        ''' Save a user without activation key when status is activation email pending '''
        self.fergus.membershipstatus = "EP"
        self.fergus.activationlinkkey = ""
        self.assertRaises(User.ImpossibleToActivate, self.fergus.save)
        
    def testSaveUpdate(self):    
        ''' Updates existing record successfully. '''
        self.fergus.firstname = "Freegus"
        self.fergus.save()
        self.assertEquals(self.fergus.firstname, "Freegus")

    def testSendActivationEmail(self):
        ''' Check that send activation email sends one email '''
        self.fergus.membershipstatus = "EP"
        self.fergus.SendActivationEmail()
        self.assertEquals(len(mail.outbox), 1)
        
    def testSendActivationEmailErroneously(self):
        ''' Try with user who does not require activation '''
        self.fergus.membershipstatus = "OK"
        self.assertRaises(User.ActivationNotPending, self.fergus.SendActivationEmail)

    def testUpdatePasswordChanges(self):    
        ''' Update password gives expected change '''
        self.assertEquals(self.fergus.CheckPassword("password"), True)
        newpassword = "heyi'mcrazy"
        self.fergus.UpdatePassword("heyi'mcrazy")
        self.assertEquals(self.fergus.CheckPassword(newpassword), True)

# Reset password key
    def testSetResetKey(self):
        ''' Sets user a random password reset key and returns said '''
        self.assertEquals(type(self.fergus.SetResetKey()) is str, True)
        self.assertEquals(len(self.fergus.SetResetKey()), 12)
        # User is still active though
        self.assert_(self.fergus.CheckPassword("password"))
        

class UserManagerTests(TestCase):

    def setUp(self):
        import sha        
        self.fergus = User.objects.create(firstname="Fergus", lastname="Ferrier", email="fff21@cam.ac.uk", password=sha.new("password").hexdigest(), membershipstatus="OK", activationlinkkey="tweed")
        self.usertwo = User.objects.create(firstname="E", lastname="Parkinson", email="eric.parkinson@hotmail.com", password=sha.new("pleaselogmein").hexdigest(), membershipstatus="EP", activationlinkkey="teddy")
        self.userthree = User.objects.create(firstname="U", lastname="Larkinson", email="u.larkinson@hotmail.com", password=sha.new("pleaselogmein").hexdigest(), membershipstatus="EP", activationlinkkey="teddy222")
        self.baduser = User.objects.create(firstname="Jecca", lastname="Bludline", email="seismics@gmail.com", password=sha.new("loggame").hexdigest(), membershipstatus="DA", activationlinkkey="beenhere")
        self.eventtag = EventTag.objects.create(name = "My Tag")
        
        # Create a college
        self.collegecat, created = GroupCategory.objects.get_or_create(name = "College", small_name = "COL", defaults = {'all_can_create': False})
        self.college = Group.objects.create(official_name = "Selwine College", category = self.collegecat)
        
        # Create or get the Pidgeon
        thePidge,created = User.objects.get_or_create(email = "thePidge@MyPidge.com", defaults={'firstname': "The", 'lastname': "PIDGE", 'password': "hey", 'membershipstatus':"OK", 'activationlinkkey': "here"})
        # We add a pidgelink
        pidgelink = StartLink.objects.create(url = "http://www.google.com", name = "Google", owner = thePidge, defaultlink = True)

    
    def testActivationNoKey(self):
        ''' No activation key given '''
        self.assertRaises(User.InvalidKey, User.objects.Activate, "")
        
    def testActivationCorrect(self):
        ''' Activation key for correct user '''
        self.assertEquals(User.objects.Activate("teddy"), self.usertwo)

    def testActivationAlreadyActivated(self):
        ''' Activation key for User that is already OK '''
        self.assertRaises(User.AlreadyActive, User.objects.Activate, "tweed")

    def testActivationNotPending(self):
        ''' Activation key for User that is neither OK nor EP '''
        self.assertRaises(User.ActivationNotPending, User.objects.Activate, "beenhere")

    
    def testAuthenticationCRSID(self):
        ''' Authenticate with CRSID '''
        self.assertEquals(self.fergus.Authenticate(), None)
        
    def testAuthenticationGood(self):
        ''' Authenticate with bona fide password '''
        self.assertEquals(self.fergus.Authenticate(password="password"), None)

    def testAuthenticationNotActivated(self):
        ''' Authenticate with EP-status user. '''
        self.assertRaises(User.NotActivated, self.usertwo.Authenticate, password="pleaselogmein") # Password
        self.assertRaises(User.NotActivated, self.userthree.Authenticate) # CRSID-credentials

    def testAuthenticationDeactivated(self):
        ''' Authenticate with DA-status user. '''
        self.assertRaises(User.Deactivated, self.baduser.Authenticate, password="loggame")
        
    def testAuthenticationBadPassword(self):
        ''' Authenticate with bad password '''
        self.assertRaises(User.DetailsError, self.fergus.Authenticate, password="password2")
    

    def testRegistrationCRSIDAndEmail(self):
        ''' Correct registration for Cambridge user with Raven and @cam.ac.uk returns user with same characteristics and activated user. '''
        testuser = {}
        testuser['firstname'] = "John"
        testuser['lastname'] = "Doe"
        testuser['email'] = "jd208@cam.ac.uk"
        testuser['crsid'] = "jd208"
        testuser['password_one'] = "helloworld"
        testuser['password_again'] = "helloworld"
        
        registerForm = RegisterForm(testuser, crsid=testuser['crsid'])
        self.assert_(registerForm.is_valid())
        user, activate = registerForm.save()
        
        self.assertEquals(user.firstname, testuser['firstname'])
        self.assertEquals(user.lastname, testuser['lastname'])
        self.assertEquals(user.email, testuser['email'])
        self.assertEquals(user.membershipstatus, "OK")
        self.assertEquals(len(mail.outbox), 0)
        self.assertEquals(user.CheckPassword(testuser['password_one']), True)
        
    def testRegistrationCRSID(self):
        ''' Correct registration for Cambridge user with Raven raises activation requirement. '''
        testuser = {}
        testuser['firstname'] = "John"
        testuser['lastname'] = "Doe"
        testuser['email'] = "jd208@gmail.com"
        testuser['crsid'] = "jd208"
        testuser['password_one'] = "helloworld"
        testuser['password_again'] = "helloworld"
        
        registerForm = RegisterForm(testuser, crsid = testuser['crsid'])
        self.assert_(registerForm.is_valid())
        
        user, activate = registerForm.save()
        
        cambridgeform = CambridgeRegisterForm(testuser, crsid = testuser['crsid'])
        self.assert_(cambridgeform.is_valid())
        cambridgeform.pk = user.pk
        cambridgeform.save()
        # Do this after saving cambridgeform so they get a link to their college
        user.GiveDefaults()
        
        # Requires activation
        self.assert_(activate)
        
        self.assertEquals(user.firstname, testuser['firstname'])
        self.assertEquals(user.lastname, testuser['lastname'])
        self.assertEquals(user.email, testuser['email'])
        self.assertEquals(user.person_ptr.cambridgemember.crsid, testuser['crsid'])
        self.assertEquals(bool(user.activationlinkkey), True)
        self.assertEquals(bool(len(user.links.all())), True)
        self.assertEquals(user.membershipstatus, "EP")
        self.assertEquals(len(mail.outbox), 1)
        self.assertEquals(user.CheckPassword(testuser['password_one']), True)
        
    def testRegistrationNormal(self):
        ''' Correct reg for normal user [no crsid] '''

        testuser = {}
        testuser['firstname'] = "John"
        testuser['lastname'] = "Doe"
        testuser['email'] = "jd208@gmail.com"
        testuser['password_one'] = "helloworld"
        testuser['password_again'] = "helloworld"
        
        registerForm = RegisterForm(testuser)
        self.assert_(registerForm.is_valid())
        user, activate = registerForm.save()
        
        user.GiveDefaults()
        
        # Requires activation
        self.assert_(activate)

        self.assertEquals(user.firstname, testuser['firstname'])
        self.assertEquals(user.lastname, testuser['lastname'])
        self.assertEquals(user.email, testuser['email'])
        self.assertEquals(bool(user.activationlinkkey), True)
        self.assertEquals(bool(len(user.links.all())), True)
        self.assertEquals(user.membershipstatus, "EP")
        self.assertEquals(len(mail.outbox), 1)
        self.assertEquals(user.CheckPassword(testuser['password_one']), True)
        
    def testRegistrationFull(self):
        ''' Correct reg for user who supplies all attributes '''
        

        testuser = {}
        testuser['firstname'] = "John"
        testuser['lastname'] = "Doe"
        testuser['email'] = "jd208@gmail.com"
        testuser['crsid'] = "eat12"
        testuser['password_one'] = "helloworld"
        testuser['password_again'] = "helloworld"
        testuser['subject'] = "FIN"
        testuser['type'] = "U1"
        testuser['interests'] = [self.eventtag.id]
        testuser['start_showflagstatus'] = True
        testuser['college'] = self.college.id
        
        registerForm = RegisterForm(testuser, crsid = testuser['crsid'])
        self.assert_(registerForm.is_valid())
        user, activate = registerForm.save()
        
        cambridgeform = CambridgeRegisterForm(testuser, crsid = testuser['crsid'])
        self.assert_(cambridgeform.is_valid())
        cambridgeform.pk = user.pk
        cambridgeform.save()
        
        user.GiveDefaults()
        
        # Requires activation
        self.assert_(activate)
        
        self.assertEquals(user.firstname, testuser['firstname'])
        self.assertEquals(user.lastname, testuser['lastname'])
        self.assertEquals(user.email, testuser['email'])
        self.assertEquals(user.person_ptr.cambridgemember.type, testuser['type'])
        self.assertEquals(user.person_ptr.cambridgemember.subject, testuser['subject'])
        self.assertEquals(bool(user.activationlinkkey), True)
        self.assertEquals(user.membershipstatus, "EP")
        self.assertEquals(len(mail.outbox), 1)
        self.assertEquals(len(user.groups.all()), 1)
        self.assertEquals(bool(len(user.links.all())), True)
        self.assertEquals(user.CheckPassword(testuser['password_one']), True)
    
    def testRegistrationEmailDuplicate(self):
       
        testuser = {}
        testuser['firstname'] = "John"
        testuser['lastname'] = "Doe"
        testuser['email'] = "jd208@gmail.com"
        testuser['crsid'] = "eat12"
        testuser['password_one'] = "helloworld"
        testuser['password_again'] = "helloworld"

        registerForm = RegisterForm(testuser)
        self.assertEquals(registerForm.is_valid(), True)
        registerForm.save()
        
        registerForm = RegisterForm(testuser)
        self.assertEquals(registerForm.is_valid(), False)
        self.assertEquals(registerForm.errors['email'], ["User with this Email address already exists."])
    
    def testRegistrationTwoNoCRSID(self):
        ''' Multiple users can register without a CRSID. '''
        testusera = {}
        testusera['firstname'] = "John"
        testusera['lastname'] = "Doe"
        testusera['email'] = "jd208@gmail.com"
        testusera['password_one'] = "helloworld"
        testusera['password_again'] = "helloworld"
        
        testuserb = {}
        testuserb['firstname'] = "John"
        testuserb['lastname'] = "Doe"
        testuserb['email'] = "jigglypuff@gmail.com"
        testuserb['password_one'] = "helloworld"
        testuserb['password_again'] = "helloworld"

        registerForm = RegisterForm(testusera)
        registerForm.is_valid()
        newuser, activate = registerForm.save()
        
        camForm = CambridgeRegisterForm(testusera, crsid = None, pk = newuser.pk)
        camForm.is_valid()
        camForm.save()

        registerForm = RegisterForm(testuserb)
        registerForm.is_valid()
        newuser, activate = registerForm.save()
        camForm = CambridgeRegisterForm(testuserb, crsid = None, pk = newuser.pk)
        camForm.is_valid()
        camForm.save()
    