'''
funcs.py
Functions for global Pidge use.
'''

from django.http import HttpResponseRedirect, HttpResponse
from django.core.urlresolvers import reverse


class PidgeException(Exception): pass

try:
    from functools import update_wrapper
except ImportError:
    from django.utils.functional import update_wrapper  # Python 2.3, 2.4 fallback.


def MustLogin(fn):
    """
    A decorator for a view, that redirects to the login page
    if the view is being called when the user is not signed in. 
    
    Use thus on the line before the def of a view:
    @MustLogin
    """

    def wrapped_fn(request, *args, **kwargs):
        if not request.session.get('loggedin'):
            request.session['login_message'] = "You Must Login to Access This Page"
            return HttpResponseRedirect(reverse('Users.views.Login', args=[request.path]))
        else:
            return fn(request, *args, **kwargs)
    return wrapped_fn
    
def MustLoginMessage(message):
    """
    @MustLoginMessage("You can only add comments if you are logged in.")

    """
    def decorate(view_func):
        return _CheckLogin(view_func, message)
    return decorate

class _CheckLogin(object):
    """
    Class that checks that the user is logged in, redirecting to
    the log-in page if necessary. If the test is passed, the view function
    is invoked. 
    
    We use a class here so that we can define __get__. This way, when a
    _CheckLogin object is used as a method decorator, the view function
    is properly bound to its instance.
    """
    def __init__(self, view_func, message=None):
        self.message = message
        self.view_func = view_func
        update_wrapper(self, view_func)
        
    def __get__(self, obj, cls = None):
        view_func = self.view_func.__get__(obj, cls)
        return _CheckLogin(view_func, self.message)
    
    def __call__(self, request, *args, **kwargs):
        if not request.session.get('loggedin'):
            request.session['login_message'] = self.message
            return HttpResponseRedirect(reverse('Users.views.Login', args=[request.path]))
        else:
            return self.view_func(request, *args, **kwargs)
            

def SendEmail(to, subject="", sender="", message="", headers={}):
    if not sender:
        sender = "\"%s\" <%s>" % (settings.PIDGE_FRIENDLY_NAME, settings.PIDGE_EMAIL)
    
    from django.core.mail import EmailMessage
    email = EmailMessage(subject=subject, from_email=sender, body=message, to=to, headers=headers)
    email.send()